<?php
class CMSController {
	
	protected $site_url;
	protected $common_function;
	protected $lang;
	protected $press_releases_controller;
	protected $post_controller;
	protected $page_controller;
	protected $reference_materials_controller;
	protected $speeches_controller;
	protected $circulars_controller;
	protected $insight_controller;
	protected $quarterly_bulletin_controller;
	protected $public_materials_controller;
	protected $monetary_statistic_controller;
	protected $guidelines_controller;
	protected $supervisory_policy_manual_controller;
	protected $regulatory_guides_controller;
	protected $guide_to_authorization_controller;
	protected $consultations_controller;
	protected $amlcft_controller;
	protected $legislative_council_controller;
	protected $rnm_column_controller;
	protected $research_memorandum_controller;
	protected $other_research_papers_controller;
	protected $external_publications_controller;
	protected $other_guides_controller;
	protected $sustainability_report_controller;

	public function __construct() {
		$this->site_url = get_site_url();
		$this->common_function = new CommonFunction();
		$this->lang = $this->common_function->checkSiteLang( $this->site_url );

		$this->post_controller = new PostController();
		$this->page_controller = new PageController();
		$this->annual_report_controller = new AnnualReportController();
		$this->sustainability_report_controller = new SustainabilityReportController();
		$this->half_yearly_report_controller = new HalfYearlyReportController();
		$this->regulatory_guides_controller = new RegulatoryGuidesController();
		$this->guide_to_authorization_controller = new GuideToAuthorizationController();
		$this->circulars_controller = new CircularsController();
		$this->guidelines_controller = new GuidelinesController();
		$this->supervisory_policy_manual_controller = new SupervisoryPolicyManualController();
		$this->insight_controller = new InSightController();
		$this->press_releases_controller = new PressReleaseController();
		$this->reference_materials_controller = new ReferenceMaterialsController();
		$this->speeches_controller = new SpeechesController();
		$this->public_materials_controller = new PublicMaterialsController();
		$this->quarterly_bulletin_controller = new QuarterlyBulletinController();
		// $this->demo_post_controller = new DemoPostController();
		$this->monetary_statistic_controller = new MonetaryStatisticController();
		$this->consultations_controller = new ConsultationsController();
		$this->amlcft_controller = new AMLCFTController();
		$this->legislative_council_controller = new LegislativeCouncilIssuesController();
		$this->rnm_column_controller = new RnMColumnController();
		$this->research_memorandum_controller = new ResearchMemorandumsController();
		$this->other_research_papers_controller = new OtherResearchPapersController();
		$this->external_publications_controller = new ExternalPublicationsController();
		$this->other_guides_controller = new OtherGuidesController();
    }
	
	public function get_controller( $post_type ) {
		switch( $post_type ) {
			case 'annual-report' :
				$controller = $this->annual_report_controller;
				break;
			case 'sustainability-report' :
				$controller = $this->sustainability_report_controller;
				break;
			case 'circulars' :
				$controller = $this->circulars_controller;
				break;
			case 'guidelines' :
				$controller = $this->guidelines_controller;
				break;
			case 'supervisory-p-manual' :
				$controller = $this->supervisory_policy_manual_controller;
				break;
			case 'guide-to-auth' :
				$controller = $this->guide_to_authorization_controller;
				break;
			case 'other-guides' :
				$controller = $this->other_guides_controller;
				break;
			case 'insight' :
				$controller = $this->insight_controller;
				break;
			case 'press-releases' :
				$controller = $this->press_releases_controller;
				break;
			case 'r-and-m-column' :
				$controller = $this->rnm_column_controller;
				break;
			case 'reference-materials':
				$controller = $this->reference_materials_controller;
				break;
			case 'speeches' :
				$controller = $this->speeches_controller;
				break;
			case 'public-materials' :
				$controller = $this->public_materials_controller;
				break;
			case 'quarterly-bulletin' :
				$controller = $this->quarterly_bulletin_controller;
				break;
			case 'monetary-statistic':
				$controller = $this->monetary_statistic_controller;
				break;
			case 'half-yearly-report':
				$controller = $this->half_yearly_report_controller;
				break;
			case 'consultations':
				$controller = $this->consultations_controller;
				break;
			case 'aml-cft':
				$controller = $this->amlcft_controller;
				break;
			case 'legislative-council':
				$controller = $this->legislative_council_controller;
				break;
			case 'research-memorandum':
				$controller = $this->research_memorandum_controller;
				break;
			case 'other-research-paper':
				$controller = $this->other_research_papers_controller;
				break;
			case 'external-publication':
				$controller = $this->external_publications_controller;
				break;
			case 'demo-post' :
				$controller = $this->demo_post_controller;
				break;
			case 'page' :
				$controller = $this->page_controller;
				break;
			case 'post' :
				$controller = $this->post_controller;
				break;
			default :
				// WordPress Default Post Types : attachment, revision, nav_menu_item, custom_css, customize_changeset, user_request
				return false;
				break;
		}
		return $controller;
	}
	
	public function init() {
		$this->press_releases_controller->init();
		$this->speeches_controller->init();
		$this->insight_controller->init();
		$this->rnm_column_controller->init();
		$this->annual_report_controller->init();
		$this->sustainability_report_controller->init();
		$this->half_yearly_report_controller->init();
		$this->quarterly_bulletin_controller->init();
		$this->reference_materials_controller->init();
		$this->public_materials_controller->init();
		$this->monetary_statistic_controller->init();
		$this->regulatory_guides_controller->init();
		$this->supervisory_policy_manual_controller->init();
		$this->circulars_controller->init();
		$this->guidelines_controller->init();
		$this->other_guides_controller->init();
		$this->guide_to_authorization_controller->init();
		$this->consultations_controller->init();
		$this->amlcft_controller->init();
		$this->legislative_council_controller->init();
		$this->research_memorandum_controller->init();
		$this->external_publications_controller->init();
		$this->other_research_papers_controller->init();
		// $this->demo_post_controller->init();
		$this->page_controller->init();
		$this->post_controller->init();

		unset($GLOBALS['wp_rewrite']->feeds[array_search('rss', $GLOBALS['wp_rewrite']->feeds)]);
	}

	public function get_file_type( $file ) {
		$strpos = strrpos( $file, '.' );
		$file_extension = substr( $file, ( $strpos + 1 ) );

		switch ( $file_extension ) {
			case 'pdf' :
				$filetype = 'PDF';
				break;
			case 'txt' :
				$filetype = 'TEXT';
				break;			
			case 'xls' :		
			case 'xlsx' :
				$filetype = 'EXCEL';
				break;
			default :
				$filetype = 'Other';
				break;
		}

		return $filetype;
	}

	public function admin_menu() {
		$this->remove_admin_menu();
	}

	public function update_nav_menu( $menu_id ) {
		$menu_term = get_term( $menu_id, 'nav_menu' );
		$menu = wp_get_nav_menu_items( $menu_id );
		$output_obj = [];

		foreach ( $menu as $key => $menu_item ) {
			if ( $menu_item->post_type == 'nav_menu_item' ) {
				$menu_url = $menu_item->url;
				$target = '_self';

	            switch ( $menu_item->type ) {
	             	case 'post_type':
		                $post = get_post( $menu_item->object_id );

		                $custom_model = new CustomPostType();
        				$file = new File();

						$page_redirect_bool = rwmb_get_value( 'page_redirect_bool', null, $menu_item->object_id );
						$page_redirect_url = rwmb_get_value( 'page_redirect_url', null, $menu_item->object_id );

						if ( $page_redirect_bool == 'Y') {
                    		$file_info = $file->getLinkType( $page_redirect_url );

                    		if ( $file_info['url'] != '' ) {
                    			$menu_url = $file_info['url'];
								$target = '_blank';
                    		}
						}

		                $slug = $post->post_name;
		                $target = $menu_item->target;
	             		break;
	             	case 'taxonomy':
		                $term = get_term( $menu_item->object_id, $menu_item->object);
		                $slug = $term->slug;
		                $menu_url = '';
	             		break;
	             	case 'custom':
		                $slug = '';
		                $target = $menu_item->target;
	             		break;
	             	default:
		                $slug = '';
	             		break;
	            }

	            $host = wp_parse_url( get_site_url() )['host'];

	            if ( strpos( $menu_url, $host ) !== false ) {
	            	$menu_url = wp_parse_url( $menu_url )['path'];
	            }

				$output_obj[] = [
					'ID' => $menu_item->ID,
					'title' => $menu_item->title,
					'parent' => $menu_item->menu_item_parent,
					'url' => $menu_url,
					'target' => $target,
					'status' => $menu_item->post_status,
					'slug' => isset( $slug ) ? $slug : '',
				];
			}
		}

		$file_path = $this->common_function->getSourceDirectory() . '/' . $this->lang . '/components/';
        $file_name = $menu_term->slug . '.txt';
        $this->common_function->createFile( $file_path, $file_name, $output_obj );
	}
	
	public function register_rwmb( $meta_boxes ) {
		$post_id = null;

		if ( isset( $_GET['post'] ) ) {
			$post_id = $_GET['post'];
		} else if ( isset( $_POST['post_ID'] ) ) {
			$post_id = $_POST['post_ID'];
		}

		$post_type = null;

		if ( $post_id ) {
			$post_type = get_post_type( $post_id );
		} else if ( isset( $_GET['post_type'] ) ) {
			$post_type = $_GET['post_type'];
		}

		$meta_boxes = $this->register_global_rwmb( $meta_boxes );

		if ( $post_type ) {
			switch ($post_type) {
				case "annual-report" :
					$meta_boxes = $this->annual_report_controller->register_rwmb( $meta_boxes );
					break;
				case "sustainability-report" :
					$meta_boxes = $this->sustainability_report_controller->register_rwmb( $meta_boxes );
					break;
				case 'half-yearly-report' :
					$meta_boxes = $this->half_yearly_report_controller->register_rwmb( $meta_boxes );
					break;
				case "insight" :
					$meta_boxes = $this->insight_controller->register_rwmb( $meta_boxes );
					break;
				case "circulars" :
					$meta_boxes = $this->circulars_controller->register_rwmb( $meta_boxes );
					break;
				case 'supervisory-p-manual' :
					$meta_boxes = $this->supervisory_policy_manual_controller->register_rwmb( $meta_boxes );
					break;
				case 'guidelines' :
					$meta_boxes = $this->guidelines_controller->register_rwmb( $meta_boxes );
					break;
				case 'guide-to-auth' :
					$meta_boxes = $this->guide_to_authorization_controller->register_rwmb( $meta_boxes );
					break;
				case "other-guides":
					$meta_boxes = $this->other_guides_controller->register_rwmb( $meta_boxes );
					break;
				case "press-releases" :
					$meta_boxes = $this->press_releases_controller->register_rwmb( $meta_boxes );
					break;
				case "r-and-m-column" :
					break;
				case "quarterly-bulletin" :
					$meta_boxes = $this->quarterly_bulletin_controller->register_rwmb( $meta_boxes );
					break;
				case "aml-cft":
					$meta_boxes = $this->amlcft_controller->register_rwmb( $meta_boxes );
					break;
				case "public-materials" :
					$meta_boxes = $this->public_materials_controller->register_rwmb( $meta_boxes );
					break;
				case "reference-materials" :
					$meta_boxes = $this->reference_materials_controller->register_rwmb( $meta_boxes );
					break;
				case "speeches" :
					$meta_boxes = $this->speeches_controller->register_rwmb( $meta_boxes );
					break;
				case "monetary-statistic" :
					$meta_boxes = $this->monetary_statistic_controller->register_rwmb( $meta_boxes );
					break;
				case "consultations" :
					$meta_boxes = $this->consultations_controller->register_rwmb( $meta_boxes );
					break;
				case "legislative-council" :
					$meta_boxes = $this->legislative_council_controller->register_rwmb( $meta_boxes );
					break;
				case "research-memorandum" :
					$meta_boxes = $this->research_memorandum_controller->register_rwmb( $meta_boxes );
					break;
				case "external-publication" :
					$meta_boxes = $this->external_publications_controller->register_rwmb( $meta_boxes );
					break;
				case "other-research-paper" :
					$meta_boxes = $this->other_research_papers_controller->register_rwmb( $meta_boxes );
					break;
				case "page" :
					$meta_boxes = $this->page_controller->register_rwmb( $meta_boxes );
					break;
				default : 
					$meta_boxes = $this->post_controller->register_rwmb( $meta_boxes );
					break;
			}
		}
		return $meta_boxes;
	}
	
	public function remove_admin_menu() {
		// remove_menu_page( 'edit.php?post_type=page' );
		remove_menu_page( 'edit-comments.php' );
		
		remove_meta_box( 'postcustom' , array( 'post', 'page' ) , 'normal' );
	}
	
	public function generate_post_permalink( $post_link, $id = 0 ) {
		$post = get_post( $id );
		$post_type = $post->post_type;
		$action = isset( $_GET['action'] ) ? $_GET['action'] : null;
		$link = $post_link;
		
		$controller = $this->get_controller( $post_type );

		if ( $controller ) {
			if ( method_exists( $controller, 'generate_post_permalink' ) ) {
				$link = $controller->generate_post_permalink( $post_link, $post );
			}
		}
		
		return $link;
	}

	public function update_rwmb_post_status( $post_id ) {
		$post_type = get_post_type( $post_id );
		$controller = $this->get_controller( $post_type );
		
		if ( $controller ) {
			if ( method_exists( $controller, 'update_rwmb_post_status' ) ) {
				$controller->update_rwmb_post_status( $post_id );
			}
		}
	}
	
	public function generate_group_list( $term_id, $tt_id, $taxonomy ) {
		$custom_post_type = new CustomPostType();

		$link = $custom_post_type->generate_group_list( $term_id, $tt_id, $taxonomy );
	}

	public function pre_get_posts( $query ) {
		global $preset_data;
		if ( ! is_home() && ! is_admin() && $query->is_main_query() ) {
			$query->set( 'post_type', $preset_data->get_post_types() );

			if ( isset( $query->query['page'] ) && preg_match( '/^\d+$/', $query->query['page'] ) ) {
			 	$query->set( 'page', '' );
			 	$query->set( 'name', $query->query['page'] );
			}
			if ( count($query->query) == 1 && isset($query->query['category_name'])) {
				$query->set( 'page', '' );
				$query->set( 'category_name', '' );
				$query->set( 'pagename', $query->query['category_name'] );
				$query->query['page'] = '';
				$query->query['pagename'] = $query->query['category_name'];
				unset($query->query['category_name']);
				$query->tax_query = null;
				$query->is_archive = false;
				$query->is_page = true;
				$query->is_singular = true;
				$query->is_category = false;
            }
		} else if(is_admin()){
		    $post_types = ['press-releases', 'speeches', 'insight'];

		    $post_type = $query->get('post_type');

		    if(in_array($post_type, $post_types)){
		        if($query->get('orderby') == ''){
					$query->set('orderby', array('post_date' => 'DESC', 'menu_order' => 'DESC'));
		        }
        		$query->query_vars['ignore_custom_sort'] = true;
		    }
		}
	}
	
	public function register_global_rwmb( $meta_boxes ) {
		$meta_boxes[] = array (
            'title' => 'Gloabl Setting',
            'id' => 'mb_gloabl_setting',
            'post_types' => array(
                'post',
                'page',
                'annual-report',
				'sustainability-report',
				'half-yearly-report',
                'press-releases',
                'reference-materials',
                'public-materials',
                'speeches',
                'circulars',
				'guidelines',
				'supervisory-p-manual',
				'guide-to-auth',
				'other-guides',
				'insight',
				'quarterly-bulletin',
				'monetary-statistic',
				'consultations',
				'aml-cft',
				'legislative-council',
				'r-and-m-column',
				'research-memorandum',
				'external-publication',
				'other-research-paper',
            ),
            'context' => 'normal',
            'priority' => 'high',
            'fields' => array(
                array(
					'id' => 'page_redirect_bool',
					'name' => esc_html__( 'Page Redirect', 'gloabl-metabox' ),
					'type' => 'radio',
					'placeholder' => '',
					'options' => array(
						'Y' => 'Yes',
						'N' => 'No',
					),
					'inline' => 'true',
					'std' => 'N',
				),

                array(
                    'id'     => 'page_redirect_url',
                    'type'   => 'group',
                    'clone'  => false,
                    'fields' => array(
						array(
							'id' => 'url',
							'type' => 'text',
							'name' => esc_html__( 'Page Redirect URL', 'gloabl-metabox' ),
							'placeholder' => esc_html__( 'Enter URL if need to redirect', 'gloabl-metabox' ),
							'attributes' => array(),
						),
						
						array(
							'id' => 'attach_id',
		                    'type' => 'file_advanced',
		                    'max_file_uploads' => 1,
		                    'force_delete'     => false,
		                    'max_status'       => 'false',
		                    'name' => esc_html__( 'Page Redirect File', 'gloabl-metabox' ),
		                    'max_status' => false,
		                    'max_file_uploads' => 1,
		                    'image_size' => 'thumbnail',
						),
                    ),
                ),
			),
			'revision' => true,
        );
		
		$meta_boxes[] = array (
            'title' => 'SEO Meta Setting',
            'id' => 'mb_seo_meta_setting',
            'post_types' => array(
                'post',
                'page',
                'annual-report',
				'sustainability-report',
				'half-yearly-report',
                'press-releases',
                'reference-materials',
                'public-materials',
                'speeches',
                'circulars',
				'guidelines',
				'supervisory-p-manual',
				'guide-to-auth',
				'other-guides',
                'insight',
				'quarterly-bulletin',
				'monetary-statistic',
				'consultations',
				'aml-cft',
				'legislative-council',
				'r-and-m-column',
				'research-memorandum',
				'external-publication',
				'other-research-paper',
            ),
            'context' => 'normal',
            'priority' => 'high',
            'fields' => array(                
                array (
                    'id' => 'social_media_image',
                    'type' => 'image_advanced',
                    'name' => 'Social Media Image',
                    'max_status' => false,
                    'max_file_uploads' => 1,
                    'image_size' => 'thumbnail',
                ),
			),
			'revision' => true,
		);
		
		$meta_boxes[] = array (
			'title' => 'Revision Date',
			'id' => 'mb_revision_date',
			'post_types' => array(
                'post',
                'page',
                'annual-report',
				'sustainability-report',
				'half-yearly-report',
				'press-releases',
                'reference-materials',
                'public-materials',
                'speeches',
                'circulars',
				'guidelines',
				'supervisory-p-manual',
				'guide-to-auth',
				'other-guides',
                'insight',
				'quarterly-bulletin',
				'monetary-statistic',
				'consultations',
				'aml-cft',
				'legislative-council',
				'r-and-m-column',
				'research-memorandum',
				'external-publication',
				'other-research-paper',
			),
			'context' => 'normal',
			'priority' => 'high',
			'autosave' => true,

			'fields' => array(
				array (
					'id' => 'publish_date',
					'type' => 'date',
					'name' => 'Publish Date',
					'required' => 1,
				),
				array (
					'id' => 'revision_date',
					'type' => 'date',
					'name' => 'Revision Date',
					'required' => 1,
				),
			),
			'table' => 'cms_mb_revision_date_' . $this->lang,
            'storage_type' => 'custom_table',
            'revision' => true,
		);

		$meta_boxes[] = array (
			'title' => 'Hero Banner Color/Image',
			'id' => 'mb_hero_banner_color_image',
			'post_types' => array(
                'post',
                'page',
                'annual-report',
				'sustainability-report',
                'half-yearly-report',
				'quarterly-bulletin',
                'reference-materials',
				'legislative-council',
				'monetary-statistic',
                'guidelines',
			),
			'context' => 'normal',
			'priority' => 'high',
			'autosave' => true,

			'fields' => array(
				array (
					'id' => 'hero_banner_image',
                    'type' => 'image_advanced',
                    'name' => 'Hero Banner Image',
                    'max_status' => false,
                    'max_file_uploads' => 1,
                    'image_size' => 'thumbnail',
                    'required' => false,
				),
				array (
					'id' => 'hero_banner_color',
					'type' => 'color',
					'name' => 'Hero Banner Color',
					'required' => false,
					'alpha_channel' => true,
				),
			),
            'revision' => true,
		);
		
        return $meta_boxes;
	}

	public function revise_admin_ui() {
		$str  = '<script>jQuery(document).ready(function() { jQuery(\'.inline-edit-date\').hide(); });</script>';
		$str .= '<script>jQuery(document).ready(function() { jQuery(\'.edit-timestamp\').hide(); });</script>';
		
		echo $str;
	}
}
