<?php
class SettingController {

    protected $common_function;
    protected $lang;

    public function __construct() {
        $this->common_function = new CommonFunction();
        $this->lang = $this->common_function->checkSiteLang( get_site_url() );

        add_filter( 'admin_init', array( $this, 'my_general_settings_register_fields' ) );
        add_action( 'update_option', array( $this, 'generate_setting_json') );
    }

    function my_general_settings_register_fields()
    {
        register_setting('general', 'default-social-media', null );
        register_setting('general', 'default-post-image', null );

        add_settings_field(
        	'default-social-media', 
        	'<label for="default_social_media">'.__('Default Social Media Image' , 'default_social_media' ).'</label>' , 
        	array( $this, 'social_media_fields_html' ), 
        	'general'
        );

        add_settings_field(
        	'default-post-image', 
        	'<label for="default_post_image">'.__('Default Post Image' , 'default_post_image' ).'</label>' , 
        	array( $this, 'default_image_fields_html' ), 
        	'general'
        );
    }
     
    function social_media_fields_html()
    {
        wp_enqueue_media();
        include ( get_template_directory() . '/template-parts/admin-setting/default-social-media.php' );
    }

    function default_image_fields_html()
    {
        wp_enqueue_media();
        include ( get_template_directory() . '/template-parts/admin-setting/default-image.php' );
    }

    function generate_setting_json(){
        if ( function_exists( 'get_current_screen') ) {
            if ( get_current_screen() != NULL && get_current_screen()->id != 'options' ) {
                return false;
            }
        }
        
        $options = wp_load_alloptions();
        
        $data = [
            'admin_email' => $options['admin_email'],
            'blogname' => $options['blogname'],
            'blogdescription' => $options['blogdescription'],
            'date_format' => $options['date_format'],
            'time_format' => $options['time_format'],
            'start_of_week' => $options['start_of_week'],
            'default-social-media' => $options['default-social-media'],
        ];

        $file_path = $this->common_function->getSourceDirectory() . '/' . $this->lang . '/settings';
        $file_name = 'settings.txt';
        $this->common_function->createFile( $file_path, $file_name, $data );
    }
}
