<?php
class SustainabilityReportController extends CustomPostType {

    protected $post_type;
    protected $group_slug;
    protected $post_before;

	public function __construct() {
        parent::__construct();

        $this->post_type = 'sustainability-report';
	}

	public function init() {
		$this->register_post_type();
	}
	
	public function get_model() {
		$model = new SustainabilityReport( $this->lang );
		return $model;
	}

	//register press release page
	public function register_post_type() {
        global $preset_data;
        $post_type_name = $preset_data->get_custom_post_type( $this->post_type );
        $args = [
            'name' => $post_type_name,
            'slug' => $this->post_type,
            'taxonomies' => [
                'category',
                'post_tag',
            ], 
        ];

        $this->register_custom_post_type( $args );
	}

    public function generate_json( $post ) {
        $this->generate_self_article_json( $post );
        $this->generate_all_article_json( $post );
        $this->generate_self_rss_xml( $post );
    }

    public function get_article_preview_data( $post ) {
        return $this->get_self_article_data( $post, true );
    }

    private function generate_all_article_json( $post ) {
        $post_type =  $this->post_type;
        $model = $this->get_model();

        $publish_data = $model->getAllPublishYear( $this->lang );

        $json = [];

        foreach ($publish_data as $key => $value) {
            $json[] = [
                'post_title' => $value->post_title,
                'publish_year' => $value->publish_year,
                'revision_date' => $value->revision_date,
                'report_cover' => $value->report_cover,
                'url' => wp_parse_url( get_permalink( $value->ID ) )['path'],
            ];
        }

        $file_path = $this->common_function->getSourceDirectory() . '/' . $this->lang . '/latest_articles_list';
        $file_name = "latest-{$this->post_type}.txt";
        $this->common_function->createFile( $file_path, $file_name, $json );
    }

	public function generate_post_permalink( $post_link, $post ) {
		$model = $this->get_model();
		$publish_date = $model->getPublishDateByPost( $post->ID, $this->lang );
        $publish_date = isset( $publish_date ) ? $publish_date  : null;
        $slug_list = [];

		if ( is_object( $post ) ) {
            $slug_list = $this->get_slug_list( $post->ID );
        }
        
        if (  sizeof( $slug_list ) )  {
            $new_slug = implode( '/', $slug_list );

            return str_replace( '%category%' , $new_slug , $post_link );
        }

		return str_replace( '%category%/' , date( 'Y', strtotime( $publish_date ) ) . '/' . date( 'm', strtotime( $publish_date ) ) .'/' , $post_link );
	}

    public function register_rwmb( $meta_boxes ) {
        global $preset_data;

        $meta_boxes[] = array (
            'title' => 'Information',
            'id' => 'mb_information',
            'post_types' => array(
                'sustainability-report',
            ),
            'context' => 'normal',
            'priority' => 'high',
            'autosave' => true,
            'required' => true,
            
            'fields' => array(
                array(
                    'id'     => 'report_cover',
                    'type'   => 'group',
                    'clone'  => false,
                    'fields' => array(
                        array(
                            'id'               => 'cover',
                            'name'             => 'Cover Photo',
                            'type'             => 'image_advanced',
                            'force_delete'     => false,
                            'max_file_uploads' => 1,
                            'max_status'       => false,
                            'image_size'       => 'thumbnail',
                            'required'         => true,
                        ),
                        array(
                            'id'               => 'inner_cover',
                            'name'             => 'Inner Photo',
                            'type'             => 'image_advanced',
                            'force_delete'     => false,
                            'max_file_uploads' => 1,
                            'max_status'       => false,
                            'image_size'       => 'thumbnail',
                            'required'         => true,
                        ),
                    ),
                ),
                array(
                    'id'               => 'report_description',
                    'name'             => 'Description',
                    'type'             => 'textarea',
                ),
            ),
            'table' => 'cms_mb_report_' . $this->lang,
            'storage_type' => 'custom_table',
            'revision' => true,
        );

        $meta_boxes[] = array (
            'title' => 'File Verison',
            'id' => 'mb_version',
            'post_types' => array(
                'sustainability-report',
            ),
            'context' => 'normal',
            'priority' => 'high',
            'autosave' => true,
            
            'fields' => array(
                array(
                    'id'     => 'report_version',
                    'type'   => 'group',
                    'clone'  => true,
                    'sort_clone' => true,
                    'max_clone' => 2,
                    'clone_default' => true,
                    'fields' => array(
                        array(
                            'id'               => 'version',
                            'name'             => 'Version',
                            'type'             => 'select',
                            'options'         => array(
                                'online'       => 'Online version',
                                'print'      => 'Print version',
                                'print-english'      => 'Print version - English Only',
                                'print-chinese'      => 'Print version - Chinese Only',
                            ),
                            'multiple'        => false,
                            'placeholder'     => 'Select version',
                        ),
                        array(
                            'id'               => 'issn',
                            'name'             => 'ISSN',
                            'type'             => 'text',
                            'placeholder'      => '(Optional) ISSN 0000-0000',
                        ),
                        array(
                            'id'               => 'price',
                            'name'             => 'Price',
                            'type'             => 'text',
                            'placeholder'      => '(Optional) HK$00 / US$00',
                        ),
                        array(
                            'id'               => 'file',
                            'name'             => 'File',
                            'type'             => 'file_advanced',
                            'max_file_uploads' => 1,
                            'force_delete'     => false,
                            'max_status'       => 'false',
                            'max_status' => false,
                            'max_file_uploads' => 1,
                            'image_size' => 'thumbnail',
                        ),
                    ),
                ),
            ),
            'table' => 'cms_mb_report_' . $this->lang,
            'storage_type' => 'custom_table',
            'revision' => true,
        );

        $meta_boxes[] = array (
            'title' => 'Table of Content',
            'id' => 'mb_table_of_content',
            'post_types' => array(
                'sustainability-report',
            ),
            'context' => 'normal',
            'priority' => 'high',
            'autosave' => true,
            
            'fields' => array(
                array(
                    'type' => 'custom_html',
                    'std'  => '<div style="padding-left: 15px;"><div class="rwmb-column rwmb-column-3">Type</div><div class="rwmb-column rwmb-column-4">Title</div><div class="rwmb-column rwmb-column-5">URL / File</div></div>',
                ),
                array(
                    'type' => 'divider',
                ),
                array(
                    'id'     => 'report_content',
                    'type'   => 'group',
                    'clone'  => true,
                    'sort_clone' => true,
                    'clone_default' => true,
                    'fields' => array(
                        array(
                            'id'        => 'report_subcontent',
                            'type'      => 'switch',
                            'style'     => 'square',
                            'on_label'  => 'Sub Content',
                            'off_label' => 'Content',
                            'columns' => 3,
                            'std' => 0,
                        ),
                        array(
                            'id' => 'report_title',
                            'type' => 'text',
                            'placeholder' => 'Title',
                            'columns' => 4,
                        ),
                        array(
                            'id'     => 'report_file',
                            'type'   => 'group',
                            'columns' => 5,
                            'fields' => array(
                                array(
                                    'name' => '',
                                    'id' => 'url',
                                    'type' => 'text',
                                    'placeholder' => 'URL, leave it blank if using file',
                                    'attributes' => array(),
                                ),
                                array(
                                    'type' => 'custom_html',
                                    'std'  => '<div style="height: 5px;"></div>',
                                ),
                                array(
                                    'id' => 'attach_id',
                                    'type' => 'file_advanced',
                                    'max_file_uploads' => 1,
                                    'force_delete'     => false,
                                    'max_status'       => 'false',
                                    'max_status' => false,
                                    'max_file_uploads' => 1,
                                    'image_size' => 'thumbnail',
                                ),
                            ),
                        ),
                    ),
                ),
            ),
            'table' => 'cms_mb_report_' . $this->lang,
            'storage_type' => 'custom_table',
            'revision' => true,
        );

        $meta_boxes[] = array (
            'title' => 'Others Information',
            'id' => 'mb_other_information',
            'post_types' => array(
                'sustainability-report',
            ),
            'context' => 'normal',
            'priority' => 'high',
            'autosave' => true,
            
            'fields' => array(
                array(
                    'type' => 'custom_html',
                    'std'  => '<div style="padding-left: 15px; padding-top: 5px;">Title</div>',
                    'columns' => 2,
                ),
                array(
                    'id'        => 'report_other_title',
                    'type'             => 'text',
                    'placeholder'      => '',
                    'columns' => 10,
                ),
                array(
                    'type' => 'divider',
                ),
                array(
                    'type' => 'custom_html',
                    'std'  => '<div style="padding-left: 15px;"><div class="rwmb-column rwmb-column-3">Type</div><div class="rwmb-column rwmb-column-4">Title</div><div class="rwmb-column rwmb-column-5">URL / File</div></div>',
                ),
                array(
                    'type' => 'divider',
                ),
                array(
                    'id'     => 'report_other_content',
                    'type'   => 'group',
                    'clone'  => true,
                    'sort_clone' => true,
                    'clone_default' => true,
                    'fields' => array(
                        array(
                            'id'        => 'report_subcontent',
                            'type'      => 'switch',
                            'style'     => 'square',
                            'on_label'  => 'Sub Content',
                            'off_label' => 'Content',
                            'columns' => 3,
                            'std' => 0,
                        ),
                        array(
                            'id' => 'report_title',
                            'type' => 'text',
                            'placeholder' => 'Title',
                            'columns' => 4,
                        ),
                        array(
                            'id'     => 'report_file',
                            'type'   => 'group',
                            'columns' => 5,
                            'fields' => array(
                                array(
                                    'name' => '',
                                    'id' => 'url',
                                    'type' => 'text',
                                    'placeholder' => 'URL, leave it blank if using file',
                                    'attributes' => array(),
                                ),
                                array(
                                    'type' => 'custom_html',
                                    'std'  => '<div style="height: 5px;"></div>',
                                ),
                                array(
                                    'id' => 'attach_id',
                                    'type' => 'file_advanced',
                                    'max_file_uploads' => 1,
                                    'force_delete'     => false,
                                    'max_status'       => 'false',
                                    'max_status' => false,
                                    'max_file_uploads' => 1,
                                    'image_size' => 'thumbnail',
                                ),
                            ),
                        ),
                    ),
                ),
            ),
            'table' => 'cms_mb_report_' . $this->lang,
            'storage_type' => 'custom_table',
            'revision' => true,
        );

        return $meta_boxes;
    }
	
    public function update_rwmb_post_status( $post_id ) {
        $post = get_post( $post_id );

        switch ($post->post_status) {
            case 'draft':
                // no break;
            case 'pending':
                // no break;
            case 'future':
                // no break;
                break;
            case 'trash':
                $this->generate_json( $post );
                break;
            case 'publish':
                $this->generate_json( $post );
                break;
            default:
                break;
        }
    }
}
