﻿<?php
class CustomPostType {
	protected $common_function;
	protected $lang;
	protected $site_url;
    protected $file;
	
    public function __construct() {
		$this->common_function = new CommonFunction();
        $this->file = new File();
		$this->full_url = $_SERVER['REQUEST_URI'];
		$this->site_url = get_site_url();
        $this->lang = $this->common_function->checkSiteLang( $this->site_url );
    }
    
    protected function create_mb_custom_table( $table_name, $table_args, $table_pk ) {
        MB_Custom_Table_API::create( $table_name, $table_args, $table_pk );
    }

    protected function get_group_list( $post_id, $slug ) {
        $list = [];

        $terms = wp_get_post_terms( $post_id, $slug );
            
        foreach ( $terms as $key => $value ) {
            $list[] = [
                'name' => $value->name,
                'slug' => $value->slug,
                'id' => $value->term_id,
            ];

            $childen = get_term_children( $value->term_id, $value->taxonomy );
            foreach ( $childen as $child => $value ) { 
                $list[] = [
                    'name' => get_term( $value )->name,
                    'slug' => get_term( $value )->slug,
                    'id' => get_term( $value )->term_id,
                ];
            }
        }

        return $list;
    }

    protected function get_category_path( $post_id ) {
        $categories_slugs = $this->get_slug_list( $post_id, 'category' );

        $category_slug_str = implode( '/', $categories_slugs );
        $category_path = '/' . $this->lang . '/' . $category_slug_str;

        return $category_path;
    }

    protected function get_social_media_image( $post_id ) {
        $social_media_image = rwmb_get_value( 'social_media_image', null, $post_id );

        if ($social_media_image) {
            return reset($social_media_image)['full_url'];
        } else {
            return '';
        }
    }

    protected function get_post_global_setting( $post_id, $args = [] ) {
        $meta_obj = [];
        $opts = [
            [
                'index' => 'redirect_state',
                'meta_name' => 'page_redirect_bool',
            ],
            [
                'index' => 'redirect_url',
                'meta_name' => 'page_redirect_url',
            ],
        ];

        if ( sizeof( $args ) > 0 ) {
            $opts = $args;
        }

        foreach( $opts as $key => $obj ) {
            $meta_value = rwmb_get_value( $obj['meta_name'], null, $post_id );

            switch ( $obj['meta_name'] ) {
                case "page_redirect_url":
                    $file_info = $this->file->getLinkType( $meta_value );
                    $meta_obj[] = [
                        "{$obj['index']}" => $file_info['url'],
                    ];
                    $meta_obj[] = [
                        "file_info" =>  $file_info,
                    ];
                    break;
                default:
                    $meta_obj[] = [
                        "{$obj['index']}" =>  $meta_value,
                    ];
                    break;
            }
        }

        return $meta_obj;
    }

    public function get_slug_list( $post_id, $taxonomy = 'category' ) {
        $slug_list = [];
        $terms = get_the_category( $post_id );

        if( $terms ) {
            foreach ( $terms as $key => $value ) {
                $slug_list[$key] = [];

                $this->get_term_parent($slug_list[$key], $value->term_id, $taxonomy);

                $slug_list[$key][] = $value->slug;
            }
        }

        return count( $slug_list ) > 0 ? $slug_list[0] : []; // return first slug list
    }

    private function get_term_parent(&$slug_list, $term_id, $taxonomy) {
        $parent = get_term_by('id', $term_id, $taxonomy);

        if ($parent->parent != '0') {
            $slug_list[] = $this->get_term_parent($slug_list, $parent->parent, $taxonomy);
        }

        return $parent->slug;
    }

	public function get_the_tags( $post_id ) {
		$tags = [];
		$tags_obj = get_the_tags( $post_id );

		if ( $tags_obj ) {
			foreach ( $tags_obj as $key => $value ) {
				$tags[] = $value->name;
			}
		}
		
		if ( sizeof( $tags ) > 0 ) {
			sort( $tags );
		}

		return $tags;
	}

    protected function register_custom_post_type( $opts ) {
        global $preset_data;
        $translation = $preset_data->get_cms_words($this->lang);
        $show_in_nav_menus = isset($opts['show_in_nav_menus']) ? $opts['show_in_nav_menus'] : false;

        $args = array (
			'label' => esc_html__( $opts['name'], 'hkma-custom-post-type' ),
			'labels' => array(
				'menu_name' => esc_html__( $opts['name'], 'hkma-custom-post-type' ),
				'name_admin_bar' => esc_html__( $opts['name'], 'hkma-custom-post-type' ),
				'add_new' => esc_html__( $translation['add-new'], 'hkma-custom-post-type' ),
				'add_new_item' => esc_html__( "{$translation['add-new']} {$opts['name']}", 'hkma-custom-post-type' ),
				'new_item' => esc_html__( "{$translation['new']} {$opts['name']}", 'hkma-custom-post-type' ),
				'edit_item' => esc_html__( "{$translation['edit']} {$opts['name']}", 'hkma-custom-post-type' ),
				'view_item' => esc_html__( "{$translation['view']} {$opts['name']}", 'hkma-custom-post-type' ),
				'update_item' => esc_html__( "{$translation['update']} {$opts['name']}", 'hkma-custom-post-type' ),
				'all_items' => esc_html__( "{$translation['all']} {$opts['name']}", 'hkma-custom-post-type' ),
				'search_items' => esc_html__( "{$translation['search']} {$opts['name']}", 'hkma-custom-post-type' ),
				'parent_item_colon' => esc_html__( "{$translation['parent']} {$opts['name']}", 'hkma-custom-post-type' ),
				'not_found' => esc_html__( "{$translation['no']} {$opts['name']}", 'hkma-custom-post-type' ),
				'not_found_in_trash' => esc_html__( "{$translation['no']} {$opts['name']}", 'hkma-custom-post-type' ),
				'name' => esc_html__( "{$opts['name']}", 'hkma-custom-post-type' ),
				'singular_name' => esc_html__( $opts['name'], 'hkma-custom-post-type' ),
			),
			'public' => true,
			'exclude_from_search' => false,
			'publicly_queryable' => true,
			'show_ui' => true,
			'show_in_nav_menus' => $show_in_nav_menus,
			'show_in_admin_bar' => false,
			'show_in_rest' => false,
			'capability_type' => 'post',
			'hierarchical' => false,
			'has_archive' => true,
			'query_var' => true,
			'can_export' => true,
			'rewrite_no_front' => false,
			'supports' => array(
				'title',
				'editor',
				'revisions',
				'excerpt',
                'page-attributes',
			),
			'taxonomies' => $opts['taxonomies'],
			'rewrite' => array( 'slug' => '%category%','with_front' => true, ),
		);

		register_post_type( $opts['slug'], $args );
    }

    protected function register_custom_taxonomy( $opts ) {
        $args = array (
			'label' => esc_html__( $opts['name'], 'hkma-custom-taxonomy' ),
			'labels' => array(
				'menu_name' => esc_html__( $opts['name'], 'hkma-custom-taxonomy' ),
				'all_items' => esc_html__( "All {$opts['name']}", 'hkma-custom-taxonomy' ),
				'edit_item' => esc_html__( "Edit {$opts['name']}", 'hkma-custom-taxonomy' ),
				'view_item' => esc_html__( "View {$opts['name']}", 'hkma-custom-taxonomy' ),
				'update_item' => esc_html__( "Update {$opts['name']}", 'hkma-custom-taxonomy' ),
				'add_new_item' => esc_html__( "Add new {$opts['name']}", 'hkma-custom-taxonomy' ),
				'new_item_name' => esc_html__( "New {$opts['name']}", 'hkma-custom-taxonomy' ),
				'parent_item' => esc_html__( "Parent {$opts['name']}", 'hkma-custom-taxonomy' ),
				'parent_item_colon' => esc_html__( "Parent {$opts['name']}:", 'hkma-custom-taxonomy' ),
				'search_items' => esc_html__( "Search {$opts['name']}", 'hkma-custom-taxonomy' ),
				'popular_items' => esc_html__( "Popular {$opts['name']}", 'hkma-custom-taxonomy' ),
				'separate_items_with_commas' => esc_html__( "Separate {$opts['name']} with commas", 'hkma-custom-taxonomy' ),
				'add_or_remove_items' => esc_html__( "Add or remove {$opts['name']}", 'hkma-custom-taxonomy' ),
				'choose_from_most_used' => esc_html__( "Choose most used {$opts['name']}", 'hkma-custom-taxonomy' ),
				'not_found' => esc_html__( "No {$opts['name']} found", 'hkma-custom-taxonomy' ),
				'name' => esc_html__( $opts['name'], 'hkma-custom-taxonomy' ),
				'singular_name' => esc_html__( $opts['name'], 'hkma-custom-taxonomy' ),
			),
			'public' => true,
			'show_ui' => true,
			'show_in_menu' => true,
			'show_in_nav_menus' => true,
			'show_tagcloud' => true,
			'show_in_quick_edit' => true,
			'show_admin_column' => false,
			'show_in_rest' => false,
			'hierarchical' => true,
			'query_var' => true,
			'sort' => false,
			'rewrite_no_front' => false,
			'rewrite_hierarchical' => false,
			'rewrite' => true,
		);

		register_taxonomy( $opts['slug'], $opts['post_type'], $args );
    }

    public function generate_group_list( $term_id, $tt_id, $taxonomy ) {
        $terms = [];
		$args = [
			'taxonomy' => $taxonomy,
			'hide_empty' => false,
		];

		$temp_terms = get_terms( $args );

		foreach ( $temp_terms as $key => $value ) {
            $disable = get_term_meta( $value->term_id, 'disable', false );

            $hidden = false;

            if ( count( $disable ) > 0 ) {
                if ( $disable[0] == 1 ) {
                    $hidden = true;
                }
            }
            
			$terms[] = [
				'id' => $value->term_id,
				'name' => $value->name,
				'slug' => $value->slug,
				'parent' => $value->parent,
                'hidden' => $hidden,
                'count' => $value->count,
			];
		}

		$file_path = $this->common_function->getSourceDirectory() . '/' . $this->lang . '/group-list';
		$file_name = $taxonomy.'-list.txt';
        $this->common_function->createFile( $file_path, $file_name, $terms );
        
        return ( $file_path . '/' . $file_name );
    }

    protected function generate_self_article_json( $post, $args = [] ) {
    	$data = $this->get_self_article_data( $post, false );

    	foreach ($args as $key => $value) {
    		$data['article'][$key] = $value;
    	}

        $file_path = $this->common_function->getSourceDirectory() . '/' . implode('/', array_slice( $data['slug_list'], 0, sizeof( $data['slug_list'] ) - 1 ) );
        $file_name = end( $data['slug_list'] ) . '.txt';

        $this->common_function->createFile( $file_path, $file_name, $data['article'] );
    }

    public function get_self_article_data( $post, $preview ) {
        $template = get_page_template_slug( $post->ID );

        $model = $this->get_model();

        $article = get_object_vars( $model->get_post( $post->ID, $this->lang, $template, $preview ) ); 

        $article['social_media_image'] = $this->get_social_media_image( $post->ID );
        $article['template_name'] = get_page_template_slug( $post->ID );
        $article['category_path'] = $this->get_category_path( $post->ID );
        $article['group'] = isset( $this->group_slug ) ? $this->get_group_list( $post->ID, $this->group_slug ) : [];
        $article['global_setting'] = $this->get_post_global_setting( $post->ID );
        $article['tags'] = $this->get_the_tags( $post->ID );
        $article['hero_banner_color'] = rwmb_get_value( 'hero_banner_color', null, $post->ID);
        $article['hero_banner_image'] = rwmb_get_value( 'hero_banner_image', null, $post->ID);
        
        $elementor_edit_mode = get_post_meta( $post->ID, '_elementor_edit_mode', true );

        if ( $elementor_edit_mode == 'builder' ) {
            $font_end = new \Elementor\FrontEnd();
            // $article['builder_content'] = $font_end->get_builder_content( $post->ID );
            $article['builder_content'] = 'elementor';
            $article['post_content'] = $font_end->get_builder_content( $post->ID );
        } else {
            $article['post_content'] = wpautop( $article['post_content'] );
        }

        $pathname = explode( '/', wp_parse_url( get_permalink( $post->ID ) )['path'] );
        array_shift( $pathname ); // remove first empty item
        $slug_list = array_filter( $pathname );

        return [ 'article' => $article, 'slug_list' => $slug_list ];
    }

    public function getCategoryFolderPath( $post_id, $dir ) {
        $pathname = wp_parse_url( get_permalink( $post_id ) )['path'];
        $pathname = ltrim($pathname, '/');
        $pathname = rtrim($pathname, '/');

        $folder_path = str_replace( '{{folder}}', $pathname, $dir );
       
        return $folder_path;
    }

    public function getArticleBetween( $post, $lang, $start, $end, $new_ealert ) {
        global $preset_data, $wpdb;

        $table = $preset_data->get_db_table( $lang );

        $post_type = $post->post_type;

        $articles = [];

        if ( $post_type == 'post' ) {
            switch ( $post->post_name ) {
                case 'monthly-statistical-bulletin':
                    $sql = "SELECT p.ID, p.post_title, p.post_name, p.post_type, p.post_status, rd.*, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                            FROM (SELECT ID, post_title, post_name, post_type, post_status, post_date, menu_order FROM {$wpdb->posts} where post_type='{$post_type}' and post_name='{$post->post_name}') p 
                            LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID 
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                            WHERE rd.revision_date <= '{$start}' AND rd.revision_date >= '{$end}' AND p.post_status='publish'
                            ORDER BY rd.revision_date DESC, p.post_date DESC, rd.ID DESC";

                    $articles = $wpdb->get_results( $sql );
 
                    foreach ( $articles as $key => $value ) {
                        if ( $value->r_bool == 'Y' ) {
                            $group = maybe_unserialize( $value );
                            $file_info = $this->file->getLinkType( $value->r_url );
                            $articles[$key]->url = $file_info['url'];
                            $value->post_title .= ' ' . $file_info['description'];
                        } else {
                            $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                        }
                    }
                    break;
                case 'tender-invitations':
                    $sql = "SELECT p.ID, p.post_title, p.post_content, p.post_name, p.post_type, p.post_status, rd.*, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                            FROM (SELECT ID, post_title, post_content, post_name, post_type, post_status, post_date, menu_order FROM {$wpdb->posts} where post_type='{$post_type}' and post_name='{$post->post_name}') p 
                            LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID 
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                            WHERE rd.revision_date <= '{$start}' AND rd.revision_date >= '{$end}' AND p.post_status='publish'
                            ORDER BY rd.revision_date DESC, p.post_date DESC, rd.ID DESC";

                    $articles = $wpdb->get_results( $sql );

                        foreach ( $articles as $key => $value ) {
                            $articles[$key]->url = '';
                        }
                    break;
                default:
                    // join-us
                    $rss_model = new RSS();
                    $category = $this->get_slug_list( $post->ID );
                    if ( $rss_model->is_category_registered( $category ) || $post->post_name == 'regtech-knowledge-hub' ) {
                        $is_regtech = false;
                        if ( $post->post_name == 'regtech-knowledge-hub' ) {
                            $category = get_category_by_slug( $post->post_name );
                            $term_id = $category->term_id;
                            $is_regtech = true;
                        } else {
                            $category = get_the_category( $post->ID );
                            $term_id = $category[0]->term_id;
                        }

                        if ( ! $is_regtech ) {
                            foreach ($category as $key => $value) {
                                if ( $value->slug == 'regtech-knowledge-hub' ) {
                                    $regtech_category = get_the_category( $post->ID );
                                    $post->post_name = $regtech_category[0]->slug;
                                    $is_regtech = true;
                                }
                            }
                        }
                        if ( $is_regtech ) {
                            $is_regtech_conditions = "AND ( t.term_id={$term_id} OR p.post_name = '{$post->post_name}' )";
                        } else {
                            $is_regtech_conditions = "AND t.term_id={$term_id}";
                        }

                        $sql = "SELECT p.ID, p.post_title, p.post_name, p.post_content, p.post_type, p.post_status, rd.publish_date, rd.revision_date, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                                FROM (SELECT ID, post_title, post_name, post_content, post_type, post_status, menu_order FROM {$wpdb->posts} where post_type='{$post_type}') p 
                                LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID  
                                LEFT JOIN {$wpdb->term_relationships} tr on tr.object_id = p.ID
                                LEFT JOIN {$wpdb->term_taxonomy} tt on tt.term_taxonomy_id = tr.term_taxonomy_id
                                LEFT JOIN {$wpdb->terms} t on t.term_id = tt.term_id
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                                WHERE p.post_status='publish' 
                                {$is_regtech_conditions}
                                ORDER BY rd.revision_date DESC, p.menu_order ASC, rd.ID DESC";

                        $articles = $wpdb->get_results( $sql );

                        foreach ( $articles as $key => $value ) {
                            if ( $value->r_bool == 'Y' ) {
                                $group = maybe_unserialize( $value );
                                $file_info = $this->file->getLinkType( $value->r_url );
                                $articles[$key]->url = $file_info['url'];
                                $value->post_title .= ' ' . $file_info['description'];
                            } else {
                                $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                            }
                        }

                        // $term = get_term_by( 'slug', $rss_model->is_category_registered( $category ), 'category' );
                        // $args = [
                        //     'numberposts' => -1,
                        //     'post_type' => $post->post_type,
                        //     'category' => $term->term_id,
                        // ];

                        // $articles = get_posts($args);
 
                        // foreach ( $articles as $key => $value ) {
                        //     $post_sql = "SELECT rd.*, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                        //         FROM {$wpdb->posts} p
                        //         LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID 
                        //         LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                        //         LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                        //         WHERE p.post_status='publish'
                        //         ORDER BY rd.revision_date DESC, p.post_date DESC, rd.ID DESC";

                        // $articles = $wpdb->get_results( $sql );
                        //     $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                        // }
                    }
                    break;
            }
        } else {
            switch ( $post_type ) {
                case 'press-releases':
                    $sql = "SELECT p.ID, p.post_title, p.post_type, p.post_status, rd.*, pm2.meta_value as pr_file_group, pm3.meta_value as date_format, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                            FROM (SELECT ID, post_title, post_type, post_status, post_date, menu_order FROM {$wpdb->posts} where post_type='{$post_type}') p 
                            LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID 
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='no_show') pm ON pm.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='pr_file_group') pm2 ON pm2.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='date_format') pm3 ON pm3.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                            WHERE rd.publish_date <= '{$start}' AND rd.publish_date >= '{$end}' AND p.post_status='publish' AND (pm.meta_value IS NULL OR pm.meta_value != '1')
                            ORDER BY rd.publish_date DESC, p.menu_order DESC, p.post_date DESC, rd.ID DESC";

                    $articles = $wpdb->get_results( $sql );

                    foreach ( $articles as $key => $value ) {
                        if ( $value->r_bool == 'Y' ) {
                            $group = maybe_unserialize( $value );
                            $file_info = $this->file->getLinkType( $value->r_url );
                            $articles[$key]->url = $file_info['url'];
                            $value->post_title .= ' ' . $file_info['description'];
                        } else {
                            $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                        }
                    }
                    break;
                case 'speeches':
                    $sql = "SELECT p.ID, p.post_title, p.post_type, p.post_status, rd.*, si.speaker_name, si.speaker_title, si.speaker_remarks, si.speaker_no_show, pm2.meta_value as date_format, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                            FROM (SELECT ID, post_title, post_type, post_status, post_date, menu_order FROM {$wpdb->posts} where post_type='{$post_type}') p 
                            LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID 
                            LEFT JOIN {$table['speaker_info']} si ON si.ID=p.ID 
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='no_show') pm ON pm.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='date_format') pm2 ON pm2.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                            WHERE rd.publish_date <= '{$start}' AND rd.publish_date >= '{$end}' AND p.post_status='publish' AND (pm.meta_value IS NULL OR pm.meta_value != '1')
                            ORDER BY rd.publish_date DESC, p.menu_order DESC, p.post_date DESC, rd.ID DESC";

                    $articles = $wpdb->get_results( $sql );

                    foreach ( $articles as $key => $value ) {
                        if ( $value->r_bool == 'Y' ) {
                            $group = maybe_unserialize( $value );
                            $file_info = $this->file->getLinkType( $value->r_url );
                            $articles[$key]->url = $file_info['url'];
                            $value->post_title .= ' ' . $file_info['description'];
                        } else {
                            $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                        }
                    }

                    break;
                case 'insight':
                    $sql = "SELECT p.ID, p.post_title, p.post_type, p.post_status, rd.*, isa.insight_author, pm2.meta_value as date_format, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                            FROM (SELECT ID, post_title, post_type, post_status, post_date, menu_order FROM {$wpdb->posts} where post_type='{$post_type}') p 
                            LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID 
                            LEFT JOIN {$table['insight_author']} isa ON isa.ID=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='no_show') pm ON pm.post_id=p.ID 
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='date_format') pm2 ON pm2.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                            WHERE rd.publish_date <= '{$start}' AND rd.publish_date >= '{$end}' AND p.post_status='publish' AND (pm.meta_value IS NULL OR pm.meta_value != '1')
                            ORDER BY rd.publish_date DESC, p.menu_order DESC, p.post_date DESC, rd.ID DESC";

                    $articles = $wpdb->get_results( $sql );

                    foreach ( $articles as $key => $value ) {
                        if ( $value->r_bool == 'Y' ) {
                            $group = maybe_unserialize( $value );
                            $file_info = $this->file->getLinkType( $value->r_url );
                            $articles[$key]->url = $file_info['url'];
                            $value->post_title .= ' ' . $file_info['description'];
                        } else {
                            $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                        }
                    }
                    break;
                case 'r-and-m-column':
                    $sql = "SELECT p.ID, p.post_title, p.post_type, p.post_status, rd.*, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                            FROM (SELECT ID, post_title, post_type, post_status FROM {$wpdb->posts} where post_type='{$post_type}') p 
                            LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID 
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                            WHERE rd.publish_date <= '{$start}' AND rd.publish_date >= '{$end}' AND p.post_status='publish' 
                            ORDER BY rd.publish_date DESC, rd.ID DESC";

                    $articles = $wpdb->get_results( $sql );

                    foreach ( $articles as $key => $value ) {
                        if ( $value->r_bool == 'Y' ) {
                            $group = maybe_unserialize( $value );
                            $file_info = $this->file->getLinkType( $value->r_url );
                            $articles[$key]->url = $file_info['url'];
                            $value->post_title .= ' ' . $file_info['description'];
                        } else {
                            $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                        }
                    }
                    break;
                case 'monetary-statistic':
                    $sql = "SELECT p.ID, p.post_title, p.post_type, p.post_status, rd.*, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                            FROM (SELECT ID, post_title, post_type, post_status FROM {$wpdb->posts} where post_type='{$post_type}') p 
                            LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID 
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                            WHERE rd.revision_date <= '{$start}' AND rd.revision_date >= '{$end}' AND p.post_status='publish' 
                            ORDER BY rd.revision_date DESC, rd.ID DESC";

                    $articles = $wpdb->get_results( $sql );

                    foreach ( $articles as $key => $value ) {
                        if ( $value->r_bool == 'Y' ) {
                            $group = maybe_unserialize( $value );
                            $file_info = $this->file->getLinkType( $value->r_url );
                            $articles[$key]->url = $file_info['url'];
                            $value->post_title .= ' ' . $file_info['description'];
                        } else {
                            $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                        }
                    }
                    break;
                case 'legislative-council':
                    $sql = "SELECT p.ID, p.post_title, p.post_type, p.post_status, rd.*, pm.meta_value as documents_group, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                            FROM (SELECT ID, post_title, post_type, post_status FROM {$wpdb->posts} where post_type='{$post_type}') p 
                            LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID 
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='documents_group') pm ON pm.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                            WHERE rd.publish_date <= '{$start}' AND rd.publish_date >= '{$end}' AND p.post_status='publish' 
                            ORDER BY rd.publish_date DESC, rd.ID DESC";

                    $articles = $wpdb->get_results( $sql );

                    foreach ( $articles as $key => $value ) {
                        $group = maybe_unserialize( $value->documents_group ); 

                        if ( gettype( $group ) == 'array' ) {
                            foreach ( $group as $key1 => $value1 ) {
                                if ( isset( $value1['documents'] ) ) {
                                    foreach ( $value1['documents'] as $key2 => $value2 ) {
                                        if ( isset( $value2['attach_id'] ) ) {
                                            $file_info = $this->file->getLinkType( [ 'attach_id' => $value2['attach_id'] ] );
                                        } else {
                                            $file_info = [
                                                'type' => '',
                                                'title' => '',
                                                'url' => '',
                                                'description' => '',
                                            ];
                                        }

                                        $group[$key1]['documents'][$key2]['file_info'] = $file_info;
                                    }
                                }
                            }
                        }

                        $articles[$key]->documents_group = $group;
                    }

                    break;
                case 'supervisory-p-manual':
                    $sql = "SELECT p.ID, p.post_title, p.post_content, p.post_type, p.post_status, rd.publish_date, rd.revision_date, pm.meta_value as reference_code, pm2.meta_value as reference_enclosed, pm3.meta_value as reference_type, pm4.meta_value as reference_status, pm5.meta_value as reference_date_format, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                            FROM (SELECT ID, post_title, post_content, post_type, post_status, menu_order FROM {$wpdb->posts} where post_type='{$post_type}') p 
                            LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID  
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_code') pm ON pm.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_enclosed') pm2 ON pm2.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_type') pm3 ON pm3.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_status') pm4 ON pm4.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_date_format') pm5 ON pm5.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                            WHERE rd.publish_date <= '{$start}' AND rd.publish_date >= '{$end}' AND p.post_status='publish' 
                            ORDER BY rd.publish_date DESC, p.menu_order ASC, rd.ID DESC";

                    $articles = $wpdb->get_results( $sql );

                    foreach ( $articles as $key => $value ) {
                        $group = maybe_unserialize( $value->reference_enclosed ); 

                        if ( gettype( $group ) == 'array' ) {
                            $key_value = [];
                            foreach ($group as $key2 => $value2) {
                                if ( isset( $value2['file'] ) ) {
                                    $file_info = $this->file->getLinkType( maybe_serialize( $value2['file'] ) );
                                } else {
                                    $file_info = [
                                        'type' => '',
                                        'title' => '',
                                        'url' => '',
                                        'description' => '',
                                    ];
                                }

                                $key_value[] = [
                                    'title' => ( isset( $value2['title'] ) ? $value2['title'] : '' ),
                                    'status' => ( isset( $value2['status'] ) ? $value2['status'] : '' ),
                                    'publish_date' => ( isset( $value2['date'] ) ? $value2['date'] : '' ),
                                    'date_format' => ( isset( $value2['date_format'] ) ? $value2['date_format'] : '' ),
                                    'file_info' => $file_info,
                                ];
                            }

                            $value->reference_enclosed = $key_value;
                        }

                        $articles[$key] = $value;

                        if ( $value->r_bool == 'Y' ) {
                            $group = maybe_unserialize( $value );
                            $file_info = $this->file->getLinkType( $value->r_url );
                            $articles[$key]->url = $file_info['url'];
                            $value->post_title .= ' ' . $file_info['description'];
                        } else {
                            if ( trim($value->post_content) == '' ) {
                                $articles[$key]->url = '';
                            } else {
                                $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                            }
                        }
                    }
                    break;
                case 'guidelines':
                    if ( $new_ealert ) {
                        $sql = "SELECT p.ID, p.post_title, p.post_content, p.post_type, p.post_status, rd.publish_date, rd.revision_date, pm.meta_value as reference_code, pm2.meta_value as reference_enclosed, pm3.meta_value as reference_date_format, pm4.meta_value as reference_remark, r_bool.meta_value as r_bool, r_url.meta_value as r_url,
                            stuff(
                                    (
                                        SELECT '#' + t.slug from {$wpdb->term_relationships} tr 
                                        LEFT JOIN {$wpdb->term_taxonomy} tt ON tt.term_taxonomy_id = tr.term_taxonomy_id
                                        LEFT JOIN {$wpdb->terms} t ON t.term_id = tt.term_id
                                        WHERE tr.object_id = p.ID and tt.taxonomy = 'group-regulatory-guides'
                                        for xml path('')
                                    ),1,1,''
                                ) as term_slug 
                                FROM (SELECT ID, post_title, post_type, post_status, post_content, menu_order FROM {$wpdb->posts} where post_type='{$post_type}') p 
                                LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID  
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_code') pm ON pm.post_id=p.ID
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_enclosed') pm2 ON pm2.post_id=p.ID
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_date_format') pm3 ON pm3.post_id=p.ID
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_remark') pm4 ON pm4.post_id=p.ID
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                                WHERE rd.publish_date <= '{$start}' AND rd.publish_date >= '{$end}' AND p.post_status='publish' 
                                ORDER BY rd.publish_date DESC, p.menu_order ASC, rd.ID DESC";

                        $articles = $wpdb->get_results( $sql );

                        foreach ( $articles as $key => $value ) {
                            if ( strpos( $value->term_slug, 'archive' ) ) {
                                continue;
                            }
                            $group = maybe_unserialize( $value->reference_enclosed ); 

                            if ( gettype( $group ) == 'array' ) {
                                $key_value = [];
                                foreach ($group as $key2 => $value2) {
                                    if ( isset( $value2['file'] ) ) {
                                        $file_info = $this->file->getLinkType( maybe_serialize( $value2['file'] ) );
                                    } else {
                                        $file_info = [
                                            'type' => '',
                                            'title' => '',
                                            'url' => '',
                                            'description' => '',
                                        ];
                                    }

                                    $key_value[] = [
                                        'title' => ( isset( $value2['title'] ) ? $value2['title'] : '' ),
                                        'status' => ( isset( $value2['status'] ) ? $value2['status'] : '' ),
                                        'publish_date' => ( isset( $value2['date'] ) ? $value2['date'] : '' ),
                                        'date_format' => ( isset( $value2['date_format'] ) ? $value2['date_format'] : '' ),
                                        'file_info' => $file_info,
                                    ];
                                }

                                $value->reference_enclosed = $key_value;
                            }

                            $articles[$key] = $value;

                            if ( $value->r_bool == 'Y' ) {
                                $group = maybe_unserialize( $value );
                                $file_info = $this->file->getLinkType( $value->r_url );
                                $articles[$key]->url = $file_info['url'];
                                $value->post_title .= ' ' . $file_info['description'];
                            } else {
                                $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                            }
                        }
                    } else {
                        $sql = "SELECT p.ID, p.post_title, p.post_content, p.post_type, p.post_status, rd.publish_date, rd.revision_date, pm.meta_value as reference_code, r_bool.meta_value as r_bool, r_url.meta_value as r_url,
                            stuff(
                                    (
                                        SELECT '#' + t.slug from {$wpdb->term_relationships} tr 
                                        LEFT JOIN {$wpdb->term_taxonomy} tt ON tt.term_taxonomy_id = tr.term_taxonomy_id
                                        LEFT JOIN {$wpdb->terms} t ON t.term_id = tt.term_id
                                        WHERE tr.object_id = p.ID and tt.taxonomy = 'group-regulatory-guides'
                                        for xml path('')
                                    ),1,1,''
                                ) as term_slug 
                                FROM (SELECT ID, post_title, post_type, post_status, post_content FROM {$wpdb->posts} where post_type IN ('circulars', 'guidelines')) p 
                                LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID  
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_code') pm ON pm.post_id=p.ID
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                                WHERE rd.publish_date <= '{$start}' AND rd.publish_date >= '{$end}' AND p.post_status='publish' 
                                ORDER BY rd.publish_date DESC, rd.ID DESC";

                        $articles = $wpdb->get_results( $sql );

                        foreach ( $articles as $key => $value ) {
                            if ( strpos( $value->term_slug, 'archive' ) ) {
                                continue;
                            }
                            if ( $value->r_bool == 'Y' ) {
                                $group = maybe_unserialize( $value );
                                $file_info = $this->file->getLinkType( $value->r_url );
                                $articles[$key]->url = $file_info['url'];
                                $value->post_title .= ' ' . $file_info['description'];
                            } else {
                                $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                            }
                        }
                    }
                    break;
                case 'circulars':
                    if ( $new_ealert ) {
                        $sql = "SELECT p.ID, p.post_title, p.post_content, p.post_type, p.post_status, rd.publish_date, rd.revision_date, cf.circular_remarks, cf.circular_file_group , pm.meta_value as reference_code, r_bool.meta_value as r_bool, r_url.meta_value as r_url,
                            stuff(
                                    (
                                        SELECT '#' + t.slug from {$wpdb->term_relationships} tr 
                                        LEFT JOIN {$wpdb->term_taxonomy} tt ON tt.term_taxonomy_id = tr.term_taxonomy_id
                                        LEFT JOIN {$wpdb->terms} t ON t.term_id = tt.term_id
                                        WHERE tr.object_id = p.ID and tt.taxonomy = 'group-regulatory-guides'
                                        for xml path('')
                                    ),1,1,''
                                ) as term_slug 
                                FROM (SELECT ID, post_title, post_content, post_type, post_status FROM {$wpdb->posts} where post_type='{$post_type}') p 
                                LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID  
                                LEFT JOIN {$table['circular_files']} cf ON cf.ID=p.ID 
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_code') pm ON pm.post_id=p.ID
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                                WHERE rd.publish_date <= '{$start}' AND rd.publish_date >= '{$end}' AND p.post_status='publish' 
                                ORDER BY rd.publish_date DESC, rd.ID DESC";

                        $articles = $wpdb->get_results( $sql );

                        foreach ( $articles as $key => $value ) {
                            if ( strpos( $value->term_slug, 'archive' ) ) {
                                continue;
                            }
                            if ( $value->r_bool == 'Y' ) {
                                $group = maybe_unserialize( $value );
                                $file_info = $this->file->getLinkType( $value->r_url );
                                $articles[$key]->url = $file_info['url'];
                                $value->post_title .= ' ' . $file_info['description'];
                            } else {
                                $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                            }
                        }
                    } else {
                        $sql = "SELECT p.ID, p.post_title, p.post_content, p.post_type, p.post_status, rd.publish_date, rd.revision_date, pm.meta_value as reference_code, r_bool.meta_value as r_bool, r_url.meta_value as r_url,
                            stuff(
                                    (
                                        SELECT '#' + t.slug from {$wpdb->term_relationships} tr 
                                        LEFT JOIN {$wpdb->term_taxonomy} tt ON tt.term_taxonomy_id = tr.term_taxonomy_id
                                        LEFT JOIN {$wpdb->terms} t ON t.term_id = tt.term_id
                                        WHERE tr.object_id = p.ID and tt.taxonomy = 'group-regulatory-guides'
                                        for xml path('')
                                    ),1,1,''
                                ) as term_slug 
                                FROM (SELECT ID, post_title, post_type, post_status, post_content FROM {$wpdb->posts} where post_type IN ('circulars', 'guidelines')) p 
                                LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID  
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_code') pm ON pm.post_id=p.ID
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                                LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                                WHERE rd.publish_date <= '{$start}' AND rd.publish_date >= '{$end}' AND p.post_status='publish' 
                                ORDER BY rd.publish_date DESC, rd.ID DESC";

                        $articles = $wpdb->get_results( $sql );

                        foreach ( $articles as $key => $value ) {
                            if ( strpos( $value->term_slug, 'archive' ) ) {
                                continue;
                            }
                            if ( $value->r_bool == 'Y' ) {
                                $group = maybe_unserialize( $value );
                                $file_info = $this->file->getLinkType( $value->r_url );
                                $articles[$key]->url = $file_info['url'];
                                $value->post_title .= ' ' . $file_info['description'];
                            } else {
                                $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                            }
                        }
                    }
                    break;
                case 'guide-to-auth':
                    $sql = "SELECT p.ID, p.post_title, p.post_content, p.post_type, p.post_status, rd.publish_date, rd.revision_date, pm.meta_value as reference_code, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                            FROM (SELECT ID, post_title, post_content, post_type, post_status, menu_order FROM {$wpdb->posts} where post_type='{$post_type}') p 
                            LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID  
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_code') pm ON pm.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                            WHERE rd.publish_date <= '{$start}' AND rd.publish_date >= '{$end}' AND p.post_status='publish' 
                            ORDER BY rd.publish_date DESC, p.menu_order ASC, rd.ID DESC";

                    $articles = $wpdb->get_results( $sql );

                    foreach ( $articles as $key => $value ) {
                        if ( $value->r_bool == 'Y' ) {
                            $group = maybe_unserialize( $value );
                            $file_info = $this->file->getLinkType( $value->r_url );
                            $articles[$key]->url = $file_info['url'];
                            $value->post_title .= ' ' . $file_info['description'];
                        } else {
                            $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                        }
                    }

                    break;
                case 'other-guides':
                    $category = get_the_category( $post->ID );
                    $term_id = $category[0]->term_id;
                    // code-of-practice
                    // explanatory-note
                    // practice-note
                    $sql = "SELECT p.ID, p.post_title, p.post_content, p.post_type, p.post_status, rd.publish_date, rd.revision_date, pm.meta_value as reference_code, pm2.meta_value as reference_enclosed, pm3.meta_value as reference_date_format, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                            FROM (SELECT ID, post_title, post_content, post_type, post_status, menu_order FROM {$wpdb->posts} where post_type='{$post_type}') p 
                            LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year FROM {$table['revision_date']}) rd ON rd.ID=p.ID  
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_code') pm ON pm.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_enclosed') pm2 ON pm2.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='reference_date_format') pm3 ON pm3.post_id=p.ID
                            LEFT JOIN {$wpdb->term_relationships} tr on tr.object_id = p.ID
                            LEFT JOIN {$wpdb->term_taxonomy} tt on tt.term_taxonomy_id = tr.term_taxonomy_id
                            LEFT JOIN {$wpdb->terms} t on t.term_id = tt.term_id
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                            WHERE p.post_status='publish' 
                            AND t.term_id={$term_id}
                            ORDER BY rd.publish_date DESC, p.menu_order ASC, rd.ID DESC";

                    $articles = $wpdb->get_results( $sql );

                    foreach ( $articles as $key => $value ) {
                        $group = maybe_unserialize( $value->reference_enclosed ); 

                        if ( gettype( $group ) == 'array' ) {
                            $key_value = [];
                            foreach ($group as $key2 => $value2) {
                                if ( isset( $value2['file'] ) ) {
                                    $file_info = $this->file->getLinkType( maybe_serialize( $value2['file'] ) );
                                } else {
                                    $file_info = [
                                        'type' => '',
                                        'title' => '',
                                        'url' => '',
                                        'description' => '',
                                    ];
                                }

                                $key_value[] = [
                                    'title' => ( isset( $value2['title'] ) ? $value2['title'] : '' ),
                                    'file_info' => $file_info,
                                ];
                            }

                            $value->reference_enclosed = $key_value;
                        }

                        $articles[$key] = $value;

                        if ( $value->r_bool == 'Y' ) {
                            $group = maybe_unserialize( $value );
                            $file_info = $this->file->getLinkType( $value->r_url );
                            $articles[$key]->url = $file_info['url'];
                            $value->post_title .= ' ' . $file_info['description'];
                        } else {
                        	if ( trim(strip_tags($value->post_content)) != '' ) {
                            	$articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                            } else {
                                $articles[$key]->url = '';
                        	}
                        }
                    }
                    break;
                case 'consultations':
                    $sql = "SELECT p.ID, p.post_title, p.post_content, p.post_type, p.post_status, rd.*, pm.meta_value as close_date, pm2.meta_value as document_group, r_bool.meta_value as r_bool, r_url.meta_value as r_url
                            FROM (SELECT ID, post_title, post_content, post_status, post_type FROM {$wpdb->posts} where post_type='{$post_type}') p 
                            LEFT JOIN {$table['revision_date']} rd ON rd.ID=p.ID 
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='consultation_close_date') pm ON pm.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='consultation_document_group') pm2 ON pm2.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_bool') r_bool ON r_bool.post_id=p.ID
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='page_redirect_url') r_url ON r_url.post_id=p.ID
                            WHERE p.post_status='publish' 
                            AND pm.meta_value >= '" . date('Y-m-d') . "'
                            ORDER BY rd.publish_date DESC, rd.ID DESC";

                    $articles = $wpdb->get_results( $sql );

                    foreach ( $articles as $key => $value ) {
                        foreach ( $value as $key1 => $value1 ) {
                            $group = maybe_unserialize( $value1 ); 

                            if ( gettype( $group ) == 'array' ) {
                                switch ( $key1 ) {
                                    case "document_group":
                                        $key_value = [];
                                        foreach ($group as $key2 => $value2) {
                                            if ( isset( $value2['consultation_file'] ) ) {
                                                $file_info = $this->file->getLinkType( maybe_serialize( $value2['consultation_file'] ) );
                                            } else {
                                                $file_info = [
                                                    'type' => '',
                                                    'title' => '',
                                                    'url' => '',
                                                    'description' => '',
                                                ];
                                            }
                                            $key_value[] = [
                                                'title' => ( isset( $value2['consultation_title'] ) ? $value2['consultation_title'] : '' ),
                                                'file_info' => $file_info,
                                            ];
                                        }
                                        $latest_articles_list[$key]->$key1 = $key_value;
                                        break;
                                    default:
                                        break;
                                }
                            }
                        }

                        if ( $value->r_bool == 'Y' ) {
                            $group = maybe_unserialize( $value );
                            $file_info = $this->file->getLinkType( $value->r_url );
                            $articles[$key]->url = $file_info['url'];
                            $value->post_title .= ' ' . $file_info['description'];
                        } else {
                            $articles[$key]->url = wp_parse_url( get_permalink( $value->ID ) )['path'];
                        }
                    }
                    break;
                case 'annual-report':
                case 'sustainability-report':
                case 'half-yearly-report':
                case 'quarterly-bulletin':
                    $sql = "SELECT MAX(p.ID) as ID, MAX(p.post_title) as post_title, MAX(p.post_type) as post_type, MAX(rd.publish_date) as publish_date, MAX(rd.revision_date) as revision_date, rd.publish_year, rd.publish_month
                            FROM (SELECT ID, post_title, post_status, post_type FROM {$wpdb->posts} 
                            where post_type IN ('quarterly-bulletin', 'annual-report', 'half-yearly-report', 'sustainability-report')) p 
                            LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year, DATEPART(MONTH, publish_date) AS publish_month FROM {$table['revision_date']}) rd ON rd.ID=p.ID 
                            WHERE rd.revision_date <= '{$start}' AND rd.revision_date >= '{$end}' 
                            AND p.post_status='publish' 
                            GROUP by p.post_type, publish_year, publish_month
                            ORDER BY MAX(rd.revision_date) DESC";

                    $articles = $wpdb->get_results( $sql );
                    break;
                case 'research-memorandum':
                case 'external-publication':
                case 'other-research-paper':
                    $sql = "SELECT p.ID, p.post_title, p.post_type, p.post_status, rd.*, pm.meta_value as publication_group
                            FROM (SELECT ID, post_title, post_type, post_status FROM {$wpdb->posts} where post_type IN ('research-memorandum', 'external-publication', 'other-research-paper')) p 
                            LEFT JOIN (SELECT * FROM {$wpdb->postmeta} WHERE meta_key='publication_group') pm ON pm.post_id=p.ID
                            LEFT JOIN (SELECT *, DATEPART(YEAR, publish_date) AS publish_year, DATEPART(MONTH, publish_date) AS publish_month FROM {$table['revision_date']}) rd ON rd.ID=p.ID 
                            WHERE rd.revision_date <= '{$start}' AND rd.revision_date >= '{$end}' 
                            AND p.post_status='publish' 
                            ORDER BY rd.revision_date DESC, rd.ID DESC";

                    $articles = $wpdb->get_results( $sql );

                    foreach ( $articles as $key => $value ) {
                        $publication_group = maybe_unserialize( $value->publication_group );

                        $articles[$key]->publication_group = [];

                        if ( gettype( $publication_group ) == 'array' ) { 
                            if ( isset( $publication_group[0]['group_title'] ) ) {
                                $publication_group = $publication_group[0];
                            }

                            $last_publish_date = $value->publish_date;

                            $new_documents_group = [];

                            foreach ($publication_group['documents_group'] as $key1 => $value1) {
                                // $last_publish_date = isset( $value1['document_date'] ) ? $value1['document_date'] : $last_publish_date;
                                if ( isset( $value1['document_file'] ) ) {
		                        $file_info = $this->file->getLinkType( $value1['document_file'] );
		                    } else {
		                        $file_info = [
		                            'type' => '',
		                            'title' => '',
		                            'url' => '',
		                            'description' => '',
		                        ];
		                    }

		                    $new_group = [];

		                    if ( $file_info['url'] != '' ) {
			                	$new_group['file_info'] = $file_info;
			                	$new_group['document_no'] = isset( $value1['document_no'] ) ? $value1['document_no'] : '';
			                	$new_group['document_date'] = isset( $value1['document_date'] ) ? $value1['document_date'] : '';
			                	$new_group['document_title'] = isset( $value1['document_title'] ) ? $value1['document_title'] : '';

			                	if ( isset( $value1['document_author_remark'] ) ) {
				                	$new_group['document_author_remark']['document_author'] = isset( $value1['document_author_remark']['document_author'] ) ? $value1['document_author_remark']['document_author'] : '';
				                	$new_group['document_author_remark']['document_remark'] = isset( $value1['document_author_remark']['document_remark'] ) ? $value1['document_author_remark']['document_remark'] : '';
			                	} else {
			                		$new_group['document_author_remark'] = [
			                			'document_author' => '',
			                			'document_remark' => ''
			                		];
			                	}
			                	if ( $new_group['document_date'] <= $start && $new_group['document_date'] >= $end ) {
			                		$new_documents_group[] = $new_group;
			                	}
		                	}
                            }

                            $publication_group['documents_group'] = array_reverse( $new_documents_group );
                            $articles[ $key ]->publication_group = $publication_group;

                            $articles[$key]->publish_date = $last_publish_date;
                        }
                    }
                    break;
            }
        }

        return $articles;
    }

    public function generate_self_rss_xml( $post ) {
        global $preset_data;
 
        include_once ( get_template_directory() . '/includes/models/RSS.class.php' );

        $new_ealert = true;

        $rss_model = new RSS();

        $category = $this->get_slug_list( $post->ID );

        if ( $rss_model->is_post_type_registered( $post->post_type ) || $rss_model->is_post_name_registered( $post->post_name ) || $rss_model->is_category_registered( $category ) ) {
            $translation = $preset_data->get_translation();

            // $model = $this->get_model();

            // $publish_date = $model->getPublishDateByPost( $post->ID, $this->lang );

            $date_end = date('Y-m-d', strtotime(date('Y-m-d') . '-1 year'));
            $date_start = date('Y-m-d');

            $data = $this->getArticleBetween( $post, $this->lang, $date_start, $date_end, $new_ealert );

            $category_path = $this->get_category_path( $post->ID );
            
            $xmlData = '';

            $final_post_type = $post->post_type;

            if ( count($data) > 0 ) {
                foreach ( $data as $key => $value ) {
                    $description = '';
                    if ( $value->post_type == 'supervisory-p-manual' ) {
                        if ( $value->url == '') {
                            continue;
                        }
                    }

                    if ( $value->post_type == 'speeches' ) {
                        if ( $this->lang == 'eng' ) {
                            $value->post_title = $value->post_title . ' by ' . $value->speaker_name . ', ' . $value->speaker_title;
                        } else {
                            $value->post_title = $value->speaker_title . $value->speaker_name . '：' . $value->post_title;
                        }
                        $final_post_type = $value->post_type;
                    }
                    if ( $value->post_type == 'insight' ) {
                        if ( $value->insight_author ) {
                            $value->post_title = $value->insight_author . $translation['content']['insight-on'] . $value->post_title;                        
                        } else {
                            $value->post_title = $value->post_title;
                        }
                        $final_post_type = $value->post_type;
                    }
                    if ( $value->post_type == 'quarterly-bulletin' ) {
                        $year_split = str_split($value->publish_year);

                        $year_str = '';

                        foreach ($year_split as $key1 => $value1) {
                            $year_str .= $translation['number_for_year'][$value1];
                        }

                        if ( $value->publish_month < 10 ) {
                            $value->publish_month = '0' . $value->publish_month;
                        }

                        $formatted_date = $translation['number_for_year']['year'];
                        
                        $formatted_date = str_replace(['{{%year%}}', '{{%month%}}'], [$year_str, $translation['month'][$value->publish_month]], $formatted_date);

                        $value->post_title = $translation['post_type'][$value->post_type] . ' (' . $formatted_date . ')';

                        $post_category_path = $this->get_category_path( $value->ID );
                        $value->url = $post_category_path . '/' . $value->publish_year . '/' . $value->publish_month . '/';

                        $final_post_type = $value->post_type;
                    }
                    if ( $value->post_type == 'annual-report' ) {
                        $post_category_path = $this->get_category_path( $value->ID );
                        $value->url = $post_category_path . '/' . $value->publish_year . '/';
                    }
					if ( $value->post_type == 'sustainability-report' ) {
                        $post_category_path = $this->get_category_path( $value->ID );
                        $value->url = $post_category_path . '/' . $value->publish_year . '/';
                    }
                    if ( $value->post_type == 'half-yearly-report' ) {
                        if ( $value->publish_month < 10 ) {
                            $value->publish_month = '0' . $value->publish_month;
                        }

                        $post_category_path = $this->get_category_path( $value->ID );
                        $value->url = $post_category_path . '/' . $value->publish_year . $value->publish_month . '/';
                    }
                    if ( $value->post_type == 'legislative-council' ) {
                        $value->url = $category_path;

                        $description = '<ul>';
                        foreach ($value->documents_group as $group_key => $group) {
                            $value->post_title = $group['title'];
                            foreach ($group['documents'] as $doc_key => $document) {
                                if (isset($document['url']) && $document['url']) {
                                    $description .= '<li><a href="' . $document['url'] . '">' . $document['filename'] . '</a></li><br>';
                                } else {
                                    $description .= '<li><a href="' . $document['file_info']['url'] . '">' . $document['filename'] . '</a></li><br>';
                                }
                            }
                        }
                        $description .= '</ul>';

                        $final_post_type = $value->post_type;
                    }
                    if ( $value->post_type == 'post' && $value->post_name == 'monthly-statistical-bulletin' ) {
                        $year_split = str_split(substr($value->publish_date, 0, 4));

                        $year_str = '';

                        foreach ($year_split as $key1 => $value1) {
                            $year_str .= $translation['number_for_year'][$value1];
                        }

                        $publish_month = substr($value->publish_date, 5, 2);

                        $formatted_date = $translation['number_for_year']['year'];
                        
                        $formatted_date = str_replace(['{{%year%}}', '{{%month%}}'], [$year_str, $translation['month'][$publish_month]], $formatted_date);

                        $value->post_title .= ' (' . $formatted_date . ')';

                        $final_post_type = $value->post_type;
                    }
                    if ( $value->post_type == 'research-memorandum' || $value->post_type == 'external-publication' || $value->post_type == 'other-research-paper' ) {
                        $post_category_path = $this->get_category_path( $value->ID );
                        $value->url = $post_category_path;

                        $value->post_title = $translation['post_type'][$value->post_type] . ' (' . substr($value->publish_date, 0, 4) . ')';
                        $final_post_type = $value->post_type;
                    }

                    switch ( $final_post_type ) {
                        case 'speeches':
                        case 'insight':
                        case 'r-and-m-column':
                        case 'press-releases':
                        case 'guidelines':
                        case 'circulars':
                        case 'legislative-council':
                        case 'consultations':
                        case 'guide-to-auth':
                        case 'supervisory-p-manual':
                        case 'other-guides':
                            $publish_date = date_create( $value->publish_date );
                            break;
                        case 'post':
                        case 'annual-report':
						case 'sustainability-report':
                        case 'half-yearly-report':
                        case 'quarterly-bulletin':
                        case 'research-memorandum':
                        case 'external-publication':
                        case 'other-research-paper':
                        case 'monetary-statistic':
                            $publish_date = date_create( $value->revision_date );
                            break;
                        default:
                            break;
                            $publish_date = date_create( $value->publish_date );
                            break;
                    }

                    $format_date = date_format( $publish_date, 'D, d M Y H:i:s' ) . ' +0800';

                    if ( substr( $value->url, 0, 1 ) == '/' ) {
                        $xml_url = 'https://' . wp_parse_url(get_site_url())['host'] . $value->url;
                    } else {
                        $xml_url = $value->url;
                    }

                    $xml_title = $value->post_title;

                    if ( $final_post_type == 'monetary-statistic' ) {
                        $xml_title = 'Daily Monetary Statistics (' . $value->publish_date . ')';
                    }

                    if ( $value->post_type == 'research-memorandum' || $value->post_type == 'other-research-paper' ) {
                        $post_category_path = $this->get_category_path( $value->ID );
                        $value->url = $post_category_path;

                        foreach ($value->publication_group['documents_group'] as $doc_key => $document) {
                            $doc_publish_date = date_create( $document['document_date'] );
                            $doc_format_date = date_format( $doc_publish_date, 'D, d M Y H:i:s' ) . ' +0800';

                            $xmlData .= '<item>';

                            if (isset($document['file_info']['url']) && $document['file_info']['url']) {
                                $xmlData .= '<title><![CDATA[' . strip_tags(str_replace('&amp;', '&', $document['document_title'])) . ' ' . $document['file_info']['description'] . ']]></title>';
                                $xmlData .= '<link><![CDATA[' . 'https://' . wp_parse_url(get_site_url())['host'] . $document['file_info']['url'] . ']]></link>' ;
                            } else {
                                $xmlData .= '<title><![CDATA[' . strip_tags(str_replace('&amp;', '&', $document['document_title'])) . ']]></title>';
                                $xmlData .= '<link><![CDATA[' . $xml_url . ']]></link>' ;
                            }

                            $xmlData .= '<pubDate><![CDATA[' . $doc_format_date .  ']]></pubDate>' . 
                                        '<description><![CDATA[' . $description .  ']]></description>' . 
                                        '<guid />' . 
                                        '<sortDate><![CDATA[' . $document['document_date'] . ']]></sortDate>' . 
                                    '</item>';
                       }
                    } else if ( $value->post_type == 'other-guides' ) {
                        if ( $xml_url ) {
                            $xmlData .= '<item>' . 
                                          '<title><![CDATA[' . trim(strip_tags($a_matches[0][0])) . ']]></title>' . 
                                          '<link><![CDATA[' . $xml_url . ']]></link>' .
                                          '<pubDate><![CDATA[' . $format_date .  ']]></pubDate>' . 
                                          '<description><![CDATA[]]></description>' . 
                                          '<guid />' . 
                                          '<sortDate><![CDATA[' . date_format($publish_date, 'Y-m-d') . ']]></sortDate>' . 
                                        '</item>';
                        }

                        if ( gettype( $value->reference_enclosed ) == 'array' ) {
                            foreach ( $value->reference_enclosed as $enkey => $envalue) {
                                if ( substr( $envalue['file_info']['url'], 0, 1 ) == '/' ) {
                                    $en_url = 'https://' . wp_parse_url(get_site_url())['host'] . $envalue['file_info']['url'];
                                } else {
                                    $en_url = $envalue['file_info']['url'];
                                }
                                $xmlData .= '<item>' . 
                                              '<title><![CDATA[' . trim(strip_tags($envalue['title'])) . ']]></title>' . 
                                              '<link><![CDATA[' . $en_url . ']]></link>' .
                                              '<pubDate><![CDATA[' . $format_date .  ']]></pubDate>' . 
                                              '<description><![CDATA[]]></description>' . 
                                              '<guid />' . 
                                              '<sortDate><![CDATA[' . date_format($publish_date, 'Y-m-d') . ']]></sortDate>' . 
                                            '</item>';
                            }
                        }
                    } else if ( isset( $value->post_name ) && $value->post_name == 'tender-invitations' ) {
                        if ( preg_match_all('/<\s*ul[^>]*>(.|\t|\r|\n)*?<\/ul>/', $value->post_content, $ul_matches) ) {
                            foreach ( $ul_matches[0] as $ul_key => $ul_value ) {
                                if ( preg_match_all('/<\s*li[^>]*>(.|\t|\r|\n)*?<\/li>/', $ul_value, $li_matches) ) {
                                   foreach ( $li_matches[0] as $li_key => $li_value ) {
                                       if ( preg_match_all('/<\s*a[^>]*>(.|\t|\r|\n)*?<\/a>/', $li_value, $a_matches) ) {
                                           $date = trim( strip_tags( str_replace( $a_matches[0][0], '', $li_value ) ) );
                                           $date = str_replace('年','-', $date);
										   $date = str_replace('月','-', $date);
										   $date = str_replace('日','', $date);

                                           preg_match_all('/href=\"(.)*?\"/', $a_matches[0][0], $url_matches);
                                           
                                           $a_match = str_replace('href="', '', $url_matches[0][0]);
                                           $a_url = str_replace('"', '', $a_match);
                                           
                                           $publish_date = date_create( $date );
                                           $format_date = date_format( $publish_date, 'D, d M Y H:i:s' ) . ' +0800';
                                           
                    if ( substr( $a_url, 0, 1 ) == '/' ) {
                        $xml_url = 'https://' . wp_parse_url(get_site_url())['host'] . $a_url;
                    } else {
                        $xml_url = $a_url;
                    }
                                           $xmlData .= '<item>' . 
                                                      '<title><![CDATA[' . trim(strip_tags($a_matches[0][0])) . ']]></title>' . 
                                                      '<link><![CDATA[' . $xml_url . ']]></link>' .
                                                      '<pubDate><![CDATA[' . $format_date .  ']]></pubDate>' . 
                                                      '<description><![CDATA[]]></description>' . 
                                                      '<guid />' . 
                                                      '<sortDate><![CDATA[' . date_format($publish_date, 'Y-m-d') . ']]></sortDate>' . 
                                                  '</item>';
                                       }
                                   }
                                }
                            }
                        }
                    } else {
                        $xmlData .= '<item>' . 
                                '<title><![CDATA[' . strip_tags(str_replace('&amp;', '&', $value->post_title)) . ']]></title>' . 
                                '<link><![CDATA[' . $xml_url . ']]></link>' . 
                                '<pubDate><![CDATA[' . $format_date .  ']]></pubDate>' . 
                                '<description><![CDATA[' . $description .  ']]></description>' . 
                                '<guid />' . 
                                '<sortDate><![CDATA[' . $value->publish_date . ']]></sortDate>' . 
                            '</item>';
                    }
                }
            }

            $xml_name = $post->post_title;
            if ( !$final_post_type == 'post') {
                $xml_name = $translation['post_type'][ $final_post_type ];
            } else if ( $rss_model->is_post_type_registered( $final_post_type ) ) {
                switch ($final_post_type) {
                    case 'annual-report':
                    case 'half-yearly-report':
                    case 'quarterly-bulletin':
                        if ( $new_ealert ) {
                            $xml_name = $translation['post_type'][ 'publications' ];
                        } else {
                            $xml_name = $translation['post_type'][ 'publications' ] . ', ' . $translation['post_type'][ 'research' ];
                        }
                        break;
                    case 'research-memorandum':
                    case 'external-publication':
                    case 'other-research-paper':
                        $xml_name = $translation['post_type'][ 'research' ];
                        break;
                    case 'monetary-statistic':
                        $xml_name = 'Daily Monetary Statistics';
                        break;
                    default: 
                        $xml_name = $translation['post_type'][ $final_post_type ];
                        break;
                }
            } else if ( $rss_model->is_post_name_registered( $post->post_name ) ) {
                $xml_name = $post->post_title;
            } else if ( $rss_model->is_category_registered( $category ) ) {
                $term = get_term_by( 'slug', $rss_model->is_category_registered( $category ), 'category' );

                $xml_name = $term->name;
            }

            foreach ($category as $key => $value) {
                if ( $value == 'regtech-knowledge-hub' ) {
                    $term = get_term_by( 'slug', 'regtech-knowledge-hub', 'category' );
                    $xml_name = $term->name;
                    break;
                }
            }

            $xml =  '<?xml version="1.0" encoding="UTF-8"?>' .
                    '<?xml-stylesheet title="XSL_formatting" type="text/xsl" href="rssxsl_data.xml" ?>' .
                    '<rss version="2.0">' .
                        '<channel>' . 
                            '<title><![CDATA[' . $translation['content']['rss-hkma'] . ': ' . strip_tags(str_replace('&amp;', '&', $xml_name)) . ']]></title>' . 
                            '<link><![CDATA[' . 'https://' . wp_parse_url(get_site_url())['host'] . $category_path . ']]></link>' . 
                            '<description><![CDATA[' . $translation['content']['rss'] . ']]></description>' . 
                            '<language><![CDATA[' . $this->lang . ']]></language>' . 
                            '<copyright><![CDATA[' . date('Y') . $translation['footer']['copy-right'] . ']]></copyright>' . 
                            '<docs />' . 
                            '<image>' . 
                                '<title><![CDATA[' . $translation['content']['rss-hkma'] . ']]></title>' . 
                                '<url><![CDATA[/' . $this->lang . '/img/logo_rss.gif]]></url>' . 
                                '<link><![CDATA[' . get_site_url() . ']]></link>' . 
                            '</image>' .
                            $xmlData . 
                        '</channel>' . 
                    '</rss>';
            
            $file_path = $this->common_function->getSourceDirectory() . '/html/' . $this->lang . '/other-information/rss';
            $file_name = '';

            $rss_prefix = 'rss_';

            if ( $post->post_type == 'post' ) {
                switch ( $post->post_name ) {
                    case 'monthly-statistical-bulletin':
                    case 'tender-invitations':
                        $file_name = $rss_prefix . $post->post_name . '.xml';
                        break;
                    case 'regtech-knowledge-hub':
                            $file_name = $rss_prefix . 'regtech-knowledge-hub.xml';
                        break;
                    default:
                        if ( end($category) == 'join-us' ) {
                            // $file_name = $rss_prefix . 'join-us.xml';
                        } else if ( end($category) == 'current-vacancies' ) {
                            $file_name = $rss_prefix . 'join-us.xml';
                        } else if ( end($category) == 'regtech-knowledge-hub' ) {
                            $file_name = $rss_prefix . 'regtech-knowledge-hub.xml';
                        } else {
                            // if ( $new_ealert ) {
                            //     $file_name = $rss_prefix . $rss_model->is_category_registered( $category ) . '.xml';
                            // } else {
                            //     $file_name = $rss_prefix . 'careers.xml';
                            // }
                        }
                        break;
                }
            } else {
                switch ( $post->post_type ) {
                    case 'speeches':
                    case 'insight':
                    case 'r-and-m-column':
                    case 'consultations':
                        $file_name = $rss_prefix . $post->post_type . '.xml';
                        break;
                    case 'press-releases':
                        $file_name = $rss_prefix . 'press-release.xml';
                        break;
                    case 'guide-to-auth':
                        $file_name = $rss_prefix . 'guide-to-authorization.xml';
                        break;
                    case 'supervisory-p-manual':
                        $file_name = $rss_prefix . 'supervisory-policy-manual.xml';
                        break;
                    case 'monetary-statistic':
                        $file_name = $rss_prefix . 'daily-monetary-statistics.xml';
                        break;
                    case 'legislative-council':
                        $file_name = $rss_prefix . 'legislative-council-issues.xml';
                        break;
                    case 'guidelines':
                    case 'circulars':
                        if ( $new_ealert ) {
                            $file_name = $rss_prefix . $post->post_type . '.xml';
                        } else {
                            $file_name = $rss_prefix . 'guidelines-and-circulars.xml';
                        }
                        break;
                    case 'other-guides':
                        $category = get_the_category( $post->ID );
                        $term_slug = $category[0]->slug;
                        $file_name = $rss_prefix . $term_slug . '.xml';
                        break;
                    case 'annual-report':
					case 'sustainability-report':
                    case 'half-yearly-report':
                    case 'quarterly-bulletin':
                        if ( $new_ealert ) {
                            $file_name = $rss_prefix . 'publications.xml';
                        } else {
                            $file_name = $rss_prefix . 'publications-and-research.xml';
                        }
                        break;
                    case 'research-memorandum':
                    case 'external-publication':
                    case 'other-research-paper':
                        $file_name = $rss_prefix . 'research.xml';
                        break;
                }
            }

            if ( $file_name != '' ) {
                $this->common_function->createXmlFile( $file_path, $file_name, $xml );
                if ( $post->post_type == 'monetary-statistic' ) {
                    $xml = str_replace('Daily Monetary Statistics', '每日金融數據', $xml);
                    $xml = str_replace('Monetary Statistics', '金融數據', $xml);
                    $xml = str_replace('/eng/', '/chi/', $xml);
                    $xml = str_replace('[eng]', '[chi]', $xml);
                    $xml = str_replace('[eng]', '[chi]', $xml);
                    $xml = str_replace('/eng]', '/chi]', $xml);
                    $xml = str_replace('HKMA:', '金管局:', $xml);

                    $this->common_function->createXmlFile( str_replace('/eng/', '/chi/', $file_path), $file_name, $xml );
                }
                // if ( $this->lang == 'chi' ) {
                //     $this->common_function->createXmlFile( str_replace('/chi/', '/gb_chi/', $file_path), $file_name, str_replace('/chi/', '/gb_chi/', $xml) );
                // }
            }
        }
    }
}
