<?php
class PresetData {

    protected $db_table;
    protected $custom_table_opts;
    protected $custom_post_type;
    protected $custom_category_group;
    protected $post_types;
    protected $translation_json;
    protected $district_json;
    protected $lang;

    public function __construct( $lang = null ) {
        $this->lang = $lang;
        $this->load_tranlation();
        $this->load_district();
        $this->define_database_table();
        $this->define_custom_post_type();
        $this->define_custom_category_group();
        $this->define_post_types();

        $this->perpare_custom_table_opts();
    }

    public function get_db_table( $lang ) {
        $data = $this->db_table;
        foreach ( $data as $key => $value ) {
            $data[$key] = $value . $lang;
        }
        return $data;
    }

    public function get_custom_post_type( $post_type = null ) {
        if ( $post_type ) {
            if ( array_key_exists( $post_type, $this->custom_post_type ) ) {
                return $this->custom_post_type[$post_type];
            }
        }
        return $this->custom_post_type;
    }

    public function get_custom_category_group( $post_type = null ) {
        if ( $post_type ) {
            if ( array_key_exists( $post_type, $this->custom_category_group ) ) {
                return $this->custom_category_group[$post_type];
            }
        }
        return $this->custom_category_group;
    }

    public function get_post_types() {
        return $this->post_types;
    }

    public function get_cms_words( $lang ) {
        return $this->translation_json[$lang]['cms_words'];
    }

    protected function define_database_table() {
        $this->db_table = [
            'side_menu' => 'cms_mb_side_menu_',
            'article_filter' => 'cms_mb_article_list_',
            'revision_date' => 'cms_mb_revision_date_',
            'speaker_info' => 'cms_mb_speaker_info_',
            'annual_report' => 'cms_mb_report_',
            'sustainability-report' => 'cms_mb_report_',
            'half_yearly_report' => 'cms_mb_report_',
            'reference_materials' => 'cms_mb_reference_materials_',
            'circular_files' => 'cms_mb_circular_files_',
            'insight_author' => 'cms_mb_insight_author_',
            'economic_financial' => 'cms_mb_economic_data_',
            'homepage' => 'cms_mb_home_',
            'public_materials' => 'cms_mb_public_materials_',
            'monetary_statistic' => 'cms_mb_monetary_statistic_',
            'demo_column_1' => 'cms_mb_demo_column_1_',
            'demo_html_1' => 'cms_mb_demo_html_1_',
        ];
    }

    protected function define_custom_post_type() {
        $this->custom_post_type = $this->translation_json[$this->lang]['post_type'];
    }

    protected function define_custom_category_group() {
        $this->custom_category_group = [            
            'regulatory-guides' => 'Group',
            // 'insight' => 'Group',
            'guidelines' => 'Title Group',
            'press-releases' => 'Group',
            'public-materials' => 'Material Group',
            'speeches' => 'Speakers',
            'quarterly-bulletin' => 'Group',
            'reference-materials' => 'Material Group',
            'aml-cft' => 'Group',
            'other-research-paper' => 'Group',
            'post' => 'Smart Consumers Category',
        ];
    }

    protected function define_post_types() {
        $this->post_types = [
            'post', 
            'page', 
            'annual-report',
            'sustainability-report',
            'insight', 
            'circulars', 
            'guidelines',
            'supervisory-p-manual',
            'guide-to-auth',
            'other-guides',
            'half-yearly-report',
            'quarterly-bulletin', 
            'r-and-m-column', 
            'reference-materials',
            'press-releases', 
            'speeches', 
            'public-materials',
            'monetary-statistic',
            'consultations',
            'aml-cft',
            'legislative-council',
            'r-and-m-column',
            'research-memorandum',
            'external-publication',
            'other-research-paper',
        ];
    }

    protected function perpare_custom_table_opts() {
        $opts = [];

        $opts['revision_date'] = [
			'publish_date'   => "DATE NOT NULL",
			'create_file_date' => "DATE NOT NULL",
        ];
    }

    public function getDefaultTopBackground() {
        return '#63798e';
    }

    public function getDefaultImage(){
        return get_option( 'default-post-image' );
    }

    protected function load_tranlation(){
        $this->translation_json = json_decode( file_get_contents( get_template_directory() . '/../../../sources/translation.txt' ) , true );
    }

    protected function load_district(){
        $this->district_json = json_decode( file_get_contents( get_template_directory() . '/../../../sources/district.txt' ) , true );
    }

    public function get_translation(){
        return $this->translation_json[$this->lang];
    }

    public function get_district_list(){
        return $this->district_json[$this->lang];
    }

    public function get_month_list(){
        return $this->translation_json[$this->lang]['month'];
    }

    public function get_week_list(){
        return $this->translation_json[$this->lang]['week'];
    }

    public function get_content(){
        return $this->translation_json[$this->lang]['content'];
    }

    public function get_header(){
        return $this->translation_json[$this->lang]['header'];
    }

    public function getDateFormat(){
        $date_format_list = [
            'chi' => 'Y年m月d日',
            'eng' => 'd M Y',
        ];

        return $date_format_list[$this->lang];
    }
}
