<?php
class SustainabilityReport{

    protected $lang;
    protected $post_type;
    protected $file;

    public function __construct( $lang ) {
        $this->lang = $lang;
        $this->post_type = "sustainability-report";
        $this->file = new File();
    }

    public function get_post( $post_id, $lang, $preview ) {
        global $preset_data, $wpdb;

        $table = $preset_data->get_db_table( $lang );

        $post = get_object_vars( get_post( $post_id ) );

        if ( $preview ) {
            $publish = "'publish', 'draft', 'pending'";
        } else {
            $publish = "'publish'";
        }

        $sql = "SELECT TOP 1 rd.publish_date, rd.revision_date, ar.report_cover, ar.report_description, ar.report_version, ar.report_content, ar.report_other_title, ar.report_other_content
        FROM (SELECT ID FROM {$wpdb->posts} where post_type='{$this->post_type}' and post_status IN ({$publish})) p 
        LEFT JOIN {$table['revision_date']} rd ON rd.ID=p.ID 
        LEFT JOIN {$table['annual_report']} ar ON ar.ID=p.ID 
        WHERE p.ID={$post_id}";

        if ( $sql != "" ) {
            $meta = $wpdb->get_results( $sql , ARRAY_A);

            if ( sizeof ( $meta ) > 0 ) {
                foreach ( $meta[0] as $key => $value ) {

                    $cms_upload_url = wp_get_upload_dir()['baseurl'];
                    $group = maybe_unserialize( $value );

                    if ( gettype( $group ) == 'array' ) { 
                        $value = [];
                        switch ( $key ) {
                            case "report_cover":
                                $value = [
                                    'cover' => str_replace( $cms_upload_url, '/media', wp_get_attachment_url( $group['cover'][0] ) ),
                                    'inner' => str_replace( $cms_upload_url, '/media', wp_get_attachment_url( $group['inner_cover'][0] ) ),
                                ];
                                break;
                            case "report_version":
                                foreach ($group as $key1 => $value1) {
                                    if ( isset( $value1['file'] ) ) {
                                        $file_info = $this->file->getLinkType( maybe_serialize( [ 'attach_id' => $value1['file'] ] ) );
                                    } else {
                                        $file_info = [
                                            'type' => '',
                                            'title' => '',
                                            'url' => '',
                                            'description' => '',
                                        ];
                                    }
                                    
                                    $value[] = [
                                        'version' => ( isset( $value1['version'] ) ? $value1['version'] : '' ),
                                        'issn' => ( isset( $value1['issn'] ) ? $value1['issn'] : '' ),
                                        'price' => ( isset( $value1['price'] ) ? $value1['price'] : '' ),
                                        'url' => $file_info['url'],
                                        'file_info' => $file_info,
                                    ];
                                }
                                break;
                            case "report_content":
                            case "report_other_content":
                                foreach ($group as $key1 => $value1) {
                                    if ( isset( $value1['report_file'] ) ) {
                                        $file_info = $this->file->getLinkType( $value1['report_file'] );
                                    } else {
                                        $file_info = [
                                            'type' => '',
                                            'title' => '',
                                            'url' => '',
                                            'description' => '',
                                        ];
                                    }

                                    $value[] = [
                                        'subcontent' => ( ( isset( $value1['report_subcontent'] ) && $value1['report_subcontent'] == '1') ? true : false ),
                                        'title' => ( isset( $value1['report_title'] ) ? $value1['report_title'] : '' ),
                                        'url' => $file_info['url'],
                                        'file_info' => $file_info,
                                    ];
                                }
                                break;
                            default:
                                break;
                        }
                    }

                    $post[$key] = $value;
                }

            }
        }
        
        return (object)$post;
    }

    public function getAllPublishYear( $lang ) {
        global $preset_data, $wpdb;

        $table = $preset_data->get_db_table( $lang );

        $sql = "SELECT p.ID, p.post_title, DATEPART(YEAR, rd.publish_date) AS publish_year, rd.revision_date, ar.report_cover
                FROM (SELECT ID, post_title, post_status FROM {$wpdb->posts} where post_type='{$this->post_type}') p 
                LEFT JOIN {$table['revision_date']} rd ON rd.ID=p.ID 
                LEFT JOIN {$table['annual_report']} ar ON ar.ID=p.ID 
                WHERE p.post_status='publish' 
                ORDER BY publish_year DESC, rd.ID DESC";

        $year_list = $wpdb->get_results( $sql );
                        
        foreach ( $year_list as $key => $value ) {
            foreach ( $value as $key1 => $value1 ) {
                $cms_upload_url = wp_get_upload_dir()['baseurl'];
                $group = maybe_unserialize( $value1 ); 

                if ( gettype( $group ) == 'array' ) {
                    switch ( $key1 ) {
                        case "report_cover":
                            $value1 = str_replace( $cms_upload_url, '/media', wp_get_attachment_url( $group['cover'][0] ) );
                            break;
                        default:
                            break;
                    }

                    $year_list[$key]->$key1 = $value1;
                }
            }
        }

        return $year_list;
    }

    public function getPublishDateByPost( $id, $lang ){
        global $preset_data, $wpdb;

        $table = $preset_data->get_db_table( $lang );

        $args = array( 'storage_type' => 'custom_table', 'table' => $table['revision_date'] );
        
        $date = rwmb_get_value( 'publish_date', $args, $id );

        return $date;
    }
}
