<?php
use \Psr\Http\Message\ServerRequestInterface as Request;
use \Psr\Http\Message\ResponseInterface as Response;

include_once 'vendor/autoload.php';

include_once 'includes/Config.class.php';
include_once 'includes/Container.class.php';

include_once 'includes/CommonFunction.class.php';

include_once 'app/controllers/MainController.php';
include_once 'app/controllers/AdvancedSearchController.php';
include_once 'app/controllers/LibSearchController.php';
include_once 'app/controllers/TermsController.php';

$config_model = new Config();
$container_model = new Container();

$container = $container_model->getContainer();
$config = $config_model->getConfig($container);

$container_model->setup($config);

// Instantiate the App object
$app = new \Slim\App($container);

// Add the TrailingSlashMiddleware
//$app->add(new \App\Middleware\TrailingSlashMiddleware());

// Press Releases API		/key-information/press-releases/api?{year|month|category|keyword|page|pagesize}
// Press Releases			/key-information/press-releases/{year}/{month}/{article}			GET		/news/PressReleasesController
// Speeches					/key-information/speeches/{year}/{month}/{article}					GET		/news/
// Forthcoming Events		/key-information/forthcoming-events/{year}/{month}/{article} 		GET		/news/
// Guidelines and Circulars	/key-information/guidelines-and-circulars/{year}{month}//{article}	GET		/news/
// Insight					/key-information/insight/{year}/{month}/{article}					GET		/news/

$app->map(['GET', 'POST'], '/{lang}/other-information/{action:advanced-search|result}[/]', 'AdvancedSearchController:main')->setName('advanced-search-main');

$app->map(['GET', 'POST'], '/{lang}/about-us/the-hkma-information-centre/library/online-catalogue/{action:libsearch|libsearch_f}[/]','LibSearchController:main')->setName('libpage');

$app->map(['GET', 'POST'], '/{lang}/data-publications-and-research/guide-to-monetary-banking-and-financial-terms[/[{terms}[/]]]', 'TermsController:main')->setName('terms-page');

$app->get('/{lang}/smart-consumers/frequently-asked-questions/[{params:.*}]', 'MainController:faq')->setName('faq');
$app->get('/{lang}/smart-consumers/personal-cross-boundary-remittance-between-hong-kong-and-mainland-china/[{params:.*}]', 'MainController:faq')->setName('faq');
$app->get('/{lang}/key-functions/international-financial-centre/ctc-faqs/[{params:.*}]', 'MainController:faq')->setName('faq');

$app->get('/{params:.*}/{file:[\w-]+\.[\w-]+}', function($request, $response, $args) {
	$abs_path = __DIR__ . '/sources/html/' . $args['params'] . '/' . $args['file'];

	if (file_exists($abs_path)) {
		if (stripos($args['file'], '.html') || stripos($args['file'], '.htm') || stripos($args['file'], '.aspx') || stripos($args['file'], '.shtml') ) {
			echo file_get_contents($abs_path);
		} else {
			// do nothing
			$filename = basename($args['file']);
			$file_extension = strtolower( substr( strrchr( $filename, '.' ), 1) );

			switch( $file_extension ) {
			    case 'gif': $ctype = 'image/gif'; break;
			    case 'png': $ctype = 'image/png'; break;
			    case 'jpeg':
			    case 'jpg': $ctype = 'image/jpeg'; break;
			    case 'xml': $ctype = 'application/xml; charset=utf-8'; break;
			    case 'css': $ctype = 'text/css'; break;
			    case 'js': $ctype = 'text/javascript'; break;
			    case 'pdf': $ctype = 'application/pdf'; break;
			    default:
			    	$ctype = 'text/html; charset=utf-8'; break;
			}
			
			header('Content-type: ' . $ctype);
		    header('Content-Length: ' . filesize($abs_path));
		    readfile($abs_path);
		    exit();
		}
	} else {
		global $container;
		$main_controller = new MainController($container);
		$params = explode('/', $args['params'] . '/' . $args['file']);
		$main_controller->softRedirection($params);
		$common_func = new CommonFunction();
		$common_func->container = $container;
		$common_func->renderNotFoundPage($request, $response, null);
	}
});

$app->get('[/{params:.*}]', 'MainController:main')->setName('main');

$app->run();